# -*- coding: utf-8 -*-
"""
Modify Expert bot let's you manipulate all fields with a config file.
"""
import json

from intelmq.lib.bot import Bot


class CopyExtraExpertBot(Bot):
    keys: list = []

    def process(self):
        event = self.receive_message()

        if "extra" in event:
            extra = json.loads(event["extra"])

            share = {key: extra[key] for key in self.keys if key in extra}
            if share:
                event.add("shareable_extra_info", share, overwrite=False)

        self.send_message(event)
        self.acknowledge_message()


BOT = CopyExtraExpertBot



"""Base for DISP client and collector tests

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

from datetime import datetime, timedelta

import requests
from requests_mock import MockerCore

from ....lib.base import OAuthAccess_APIMockMixIn

_DEFAULT_TAGS = ["_BotnetCreds", "family: Redline", "type: InfoStealer", "_Parsed"]
_DEFAULT_ASSETS = ["nic.at"]


class DISP_APIMockMixIn(OAuthAccess_APIMockMixIn):
    def setup_api_mock(
        self,
        url: str = "https://disp.example.at/v1",
        auth_token: str = "XXX",
        oauth_clientid: str = "client1",
        oauth_clientsecret: str = "secret1",
        oauth_url: str = "https://oauthip.example.at/v1",
        session: requests.Session = None,
    ):
        self._api_url = url
        self._auth_token = auth_token
        self.session = session
        self.requests = MockerCore(session=self.session)
        self.requests.start()
        self.addCleanup(self.requests.stop)

        self.setup_oauth_mock(
            oauth_clientid=oauth_clientid,
            oauth_clientsecret=oauth_clientsecret,
            oauth_url=oauth_url,
            oauth_scope="https://gateway.disp.deloitte.com/.default",
            oauth_grant_type="client_credentials",
            session=self.session,
        )

        self._incident_counter = 0
        self._incidents = []

    def create_requests_mock(self):
        # create nested requests mock to separate tested requests
        mocker = MockerCore(session=self.session, real_http=True)
        mocker.start()
        self.addCleanup(mocker.stop)
        return mocker

    def mock_request(
        self, path: str, mocker: MockerCore = None, method: str = "get", **kwargs
    ):
        mocker = mocker or self.requests
        mocking_method = getattr(mocker, method)
        mocking_method(
            f"{self._api_url}/{path}",
            request_headers={
                "Authorization": f"Bearer {self._auth_token}",
                "OAuth": self.mocked_access_token,
            },
            **kwargs,
        )

    def add_incident(
        self,
        *,
        viewed: bool = False,
        validation_date: datetime = None,
        title: str = "Credentials compromised by a botnet",
        category: str = "Information discovery",
        type_: str = "Credentials",
        tags: list = None,
        related_assets: list = None,
        evidence_id: str = None,
        other: dict = None,
    ):
        """Add mocked incident to the API responses, but only selected used fields fields"""
        incident_id = f"incident-{self._incident_counter}"
        self._incident_counter += 1
        if tags is None:
            tags = _DEFAULT_TAGS
        if related_assets is None:
            related_assets = _DEFAULT_ASSETS
        if validation_date is None:
            validation_date = datetime.utcnow() - timedelta(days=1)

        incident = {
            "id": incident_id,
            "metadata": {
                "viewed": viewed,
            },
            "validationDate": int(validation_date.timestamp() * 1000),
            "title": title,
            "category": category,
            "type": type_,
            "tags": tags,
            "relatedAssets": related_assets,
            "url": f"https://disp.app/document/incident/{incident_id}",
        }

        if evidence_id:
            incident.update(
                {
                    "evidences": [
                        {"idStoredFile": evidence_id, "name": f"{incident_id}.json.txt"}
                    ]
                }
            )
        else:
            incident.update({"evidences": []})

        if other:
            incident.update(other)

        self._incidents.append(incident)
        return incident_id, incident

    def mock_evidence(
        self,
        incident_id: str,
        file_id: str,
        data: dict = None,
        mocker: MockerCore = None,
    ):
        mocker = mocker or self.requests
        self.mock_request(
            f"incident/{incident_id}/file/{file_id}", json=data, mocker=mocker
        )

    def _mock_incidents(self, mocker: MockerCore = None, after: datetime = None):
        after = after or datetime.utcnow() - timedelta(days=7)
        mocker = mocker or self.requests
        self.mock_request(
            (
                f"incident/?query=validationDate%20%3E%20{int(after.timestamp() * 1000)}"
                "%20AND%20UNREAD&page=0&size=10"
            ),
            mocker=mocker,
            json={
                "content": self._incidents,
                "last": True,
            },
        )



"""Testing BlackKite API client

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

from unittest import TestCase

import requests

from intelmq_extensions.bots.collectors.blackkite._client import (
    BlackKiteClient,
    Output,
    Severity,
    Status,
)

from .....lib.blackkite import Category
from .base import BlackKite_APIMockMixIn


class BlackKiteClientTestCase(BlackKite_APIMockMixIn, TestCase):
    def setUp(self) -> None:
        super().setUp()
        self.config = {
            "url": "https://blackkite.example.at/v1",
            "client_id": "client1",
            "client_secret": "secret1",
        }
        self.setup_api_mock(
            **self.config,
            session=requests.Session(),
        )

        self.client = BlackKiteClient(session=self.session, page_size=10, **self.config)

    def test_get_paginated_response_1_page(self):
        self.mock_request(
            "companies/?page_number=1&page_size=10",
            json=[{"CompanyId": 1}, {"CompanyId": 2}],
            headers={"X-Total-Items": "2"},
        )

        responses = list(self.client.get_paginated("companies/"))
        self.assertEqual([{"CompanyId": 1}, {"CompanyId": 2}], responses)

    def test_get_paginated_response_3_pages(self):
        self.mock_request(
            "incident/?page_number=1&page_size=10",
            json=[{"CompanyId": 1}, {"CompanyId": 2}],
            headers={"X-Total-Items": "23"},
        )
        self.mock_request(
            "incident/?page_number=2&page_size=10",
            json=[{"CompanyId": 3}, {"CompanyId": 4}],
            headers={"X-Total-Items": "23"},
        )
        self.mock_request(
            "incident/?page_number=3&page_size=10",
            json=[{"CompanyId": 5}],
            headers={"X-Total-Items": "23"},
        )

        responses = list(self.client.get_paginated("incident/"))
        self.assertEqual(
            [
                {"CompanyId": 1},
                {"CompanyId": 2},
                {"CompanyId": 3},
                {"CompanyId": 4},
                {"CompanyId": 5},
            ],
            responses,
        )

    def test_get_status(self):
        self.mock_request("status", json={"IsValid": True})

        self.assertEqual({"IsValid": True}, self.client.status())

    def test_get_companies(self):
        self.mock_request(
            "companies?page_number=1&page_size=10",
            json=[{"CompanyId": 1}, {"CompanyId": 2}],
            headers={"X-Total-Items": "2"},
        )

        responses = list(self.client.companies())
        self.assertEqual([{"CompanyId": 1}, {"CompanyId": 2}], responses)

    def test_list_findings_default(self):
        self.mock_request(
            (
                "companies/1/findings/patchmanagement?page_number=1&page_size=10"
                "&status=Active&severity=Critical"
            ),
            headers={"X-Total-Items": "2"},
            json=[{"FindingId": 1}, {"FindingId": 2}],
        )

        result = list(self.client.list_findings("patchmanagement", company_id=1))
        self.assertEqual([{"FindingId": 1}, {"FindingId": 2}], result)

    def test_list_findings_custom_filters(self):
        self.mock_request(
            (
                "companies/1/findings/dnshealth?page_number=1&page_size=10"
                "&status=Active,Deleted&severity=Critical,High&output=Failed,Warning,Passed"
            ),
            headers={"X-Total-Items": "2"},
            json=[{"FindingId": 1}, {"FindingId": 2}],
        )

        result = list(
            self.client.list_findings(
                "dnshealth",
                company_id=1,
                statuses=[Status.ACTIVE, Status.DELETED],
                severities=[Severity.CRITICAL, Severity.HIGH],
                outputs=[Output.FAILED, Output.WARNING, Output.PASSED],
            )
        )
        self.assertEqual([{"FindingId": 1}, {"FindingId": 2}], result)

    def test_get_category_findings_ignore_output_when_not_supported(self):
        self.mock_request(
            (
                "companies/1/findings/patchmanagement?page_number=1&page_size=10"
                "&status=Active,Deleted&severity=Critical,High"
            ),
            headers={"X-Total-Items": "2"},
            json=[{"FindingId": 1}, {"FindingId": 2}],
            complete_qs=True,
        )

        result = list(
            self.client.get_findings_from_category(
                Category.PatchManagement,
                company_id=1,
                statuses=[Status.ACTIVE, Status.DELETED],
                severities=[Severity.CRITICAL, Severity.HIGH],
                outputs=[Output.FAILED, Output.WARNING, Output.PASSED],
            )
        )
        self.assertEqual([{"FindingId": 1}, {"FindingId": 2}], result)

    def test_acknowledge_finding(self):
        self.mock_request(
            "companies/1/findings/2", json={"Status": "Acknowledged"}, method="patch"
        )

        self.client.acknowledge_finding(1, 2)

        self.assertEqual(1, self.requests.call_count)



from intelmq.lib.bot import Bot


class ToLogOutput(Bot):
    def process(self):
        event = self.receive_message()
        jevent = event.to_json()
        self.logger.info("Got message %s", jevent)
        self.acknowledge_message()


BOT = ToLogOutput



# -*- coding: utf-8 -*-
"""
The descriptions give a hint about what the entry is about and is very mixed.
Most prominent description is "phishing", most of them are malware names.
More types could be mapped better, only the most obvious ones are done currently.
"""
import datetime

from intelmq.lib import utils
from intelmq.lib.bot import Bot


class MalwareDomainsParserBot(Bot):
    def is_valid_date(self, strd):
        try:
            datetime.datetime.strptime(strd, "%Y%m%d")
            return True
        except Exception:
            return False

    def process(self):
        report = self.receive_message()

        raw_report = utils.base64_decode(report.get("raw"))

        for row in raw_report.splitlines():
            row = row.rstrip()

            if row.startswith("#") or len(row) == 0:
                continue

            values = row.split("\t")[1:]

            event = self.new_event(report)

            event.add("source.fqdn", values[1])
            if values[2] == "phishing":
                event.add("classification.identifier", values[2])
                event.add("classification.type", "phishing")
            elif values[2] == "C&C":
                event.add("classification.identifier", values[2])
                event.add("classification.type", "c2-server")
            else:
                event.add("classification.identifier", values[2])
                event.add("classification.type", "malware")
            event.add("event_description.text", values[2])

            for i in range(4, len(values)):
                if self.is_valid_date(values[i]):
                    event.add(
                        "time.source",  # times are GMT, verified via email
                        values[i] + "T00:00:00+00:00",
                        overwrite=True,
                    )
                    break

            event.add("raw", row)

            self.send_message(event)
        self.acknowledge_message()


BOT = MalwareDomainsParserBot



"""Base for BlackKite tests

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import requests
from requests_mock import MockerCore

from ....lib.base import OAuthAccess_APIMockMixIn


class BlackKite_APIMockMixIn(OAuthAccess_APIMockMixIn):
    def setup_api_mock(
        self,
        url: str = "https://blackkite.example.at/v1",
        client_id: str = "client1",
        client_secret: str = "secret1",
        session: requests.Session = None,
    ):
        self._url = url
        self.session = session
        self.requests = MockerCore(session=self.session)
        self.requests.start()
        self.addCleanup(self.requests.stop)

        self.setup_oauth_mock(
            oauth_clientid=client_id,
            oauth_clientsecret=client_secret,
            oauth_url=f"{url}/oauth/token",
            session=self.session,
        )

    def mock_request(
        self, path: str, mocker: MockerCore = None, method: str = "get", **kwargs
    ):
        mocker = mocker or self.requests
        mocking_method = getattr(mocker, method)
        mocking_method(
            f"{self._url}/{path}",
            request_headers={
                "Authorization": f"Bearer {self.mocked_access_token}",
            },
            **kwargs,
        )



"""Test data for parser tests. Extracted for readiness

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

DEFAULT_COMPANY = {
    "CompanyId": 1,
    "CompanyName": "Austrian National CERT",
    "DomainName": "cert.at",
}

PATCH_MANAGEMENT = {
    "FindingId": 1,
    "Status": "Active",
    "Severity": "Critical",
    "ControlId": "PATCH-001",
    "Domain": "example.cert.at",
    "IpAddress": "127.0.0.1",
    "ProductName": "openssh_/5.9",
    "Cpes": [
        "cpe:2.3:a:openbsd:openssh:7.6:p1:*:*:*:*:*:*",
        "cpe:2.3:a:openbsd:openssh:7.6:-:*:*:*:*:*:*",
    ],
    "PublishDate": "2023-08-02T07:30:44.38",
    "CvssScore": 9.8,
    "CwssScore": None,
    "CveId": "CVE-2023-38408",
    "CweId": "CWE-428",
    "Detail": (
        "The PKCS#11 feature in ssh-agent in OpenSSH before 9.3p2 has an insufficiently "
        "trustworthy search path, leading to remote code execution if an agent is forwarded "
        "to an attacker-controlled system. (Code in /usr/lib is not necessarily safe for "
        "loading into ssh-agent.) NOTE: this issue exists because of an incomplete fix for"
        " CVE-2016-10009."
    ),
    "References": ["https://nvd.nist.gov/vuln/detail/CVE-2023-38408"],
    "FindingDate": "2023-08-28T13:37:22.717",
    "LastCheckDate": "2023-10-03T11:48:54.257",
    "Ticket": None,
}

APPLICATION_SECURITY = {
    "FindingId": 658443,
    "Status": "Active",
    "Severity": "Critical",
    "ControlId": "APPSEC-014",
    "Domain": "example.cert.at",
    "Output": "Failed",
    "Title": "Cleartext Transmission of Sensitive Information",
    "Detail": "Not encrypted communication on xxx.",
    "FindingDate": "2023-08-28T13:37:22.717",
    "LastCheckDate": "2023-10-03T11:48:58.937",
    "Ticket": None,
}

CREDENTIAL_MANAGEMENT = {
    "FindingId": 44823698,
    "Status": "Active",
    "Severity": "Low",
    "ControlId": "LEAK-003",
    "Domain": "cert.at",
    "EmailorUsername": "user@example.cert.at",
    "Source": "cert_com_leak",
    "LeakDate": "2022-09-22T00:00:00",
    "LeakInfo": "****",
    "PasswordType": "PLAIN",
    "Ticket": None,
}



# -*- coding: utf-8 -*-
"""
Create RTIR reports for data without, per feed.code

https://github.com/certat/intelmq/issues/53#issuecomment-235338136

evlist = get all open events where report_id IS NULL
foreach distinct feed.codes in evlist
   report_data = create a zipped json file (minus raw attribute) with events from current feed.code
   create report with attachment report_data
   set report_id for the used events to newly created report

TODO: Non-batch mode
"""
import datetime
import io
import json
import sys
import time
import zipfile

from intelmq_extensions.cli import lib


class IntelMQCLIContoller(lib.IntelMQCLIContollerTemplate):
    appname = "intelmqcli_create_reports"
    retval = 0

    def run(self, args: list):
        self.parser.add_argument(
            "-l", "--list-feeds", action="store_true", help="List all open feeds"
        )
        self.parser.add_argument(
            "--internal-notify",
            action="store_true",
            help=(
                "Create open incident reports for further manual work. "
                "Ignore the notify and abuse contact fields filtering; "
                "do not set IntelMQ as ticket owner."
            ),
        )

        self.setup(args)
        self.connect_database()

        if self.args.list_feeds:
            self.execute(lib.QUERY_OPEN_FEEDCODES)
            for row in self.cur.fetchall():
                if row["feed.code"]:
                    print(row["feed.code"])
            return 0

        if not self.rt.login():
            self.logger.error(
                "Could not login as {} on {}.".format(
                    self.config["rt"]["user"], self.config["rt"]["uri"]
                )
            )
            return 2
        else:
            self.logger.info(
                "Logged in as {} on {}.".format(
                    self.config["rt"]["user"], self.config["rt"]["uri"]
                )
            )

        feedcodes_query = lib.QUERY_OPEN_FEEDCODES
        events_query = lib.QUERY_OPEN_EVENTS_BY_FEEDCODE
        ticket_kwargs = dict(Owner=self.config["rt"]["user"])
        if self.args.internal_notify:
            feedcodes_query = lib.INTERNAL_QUERY_OPEN_FEEDCODES
            events_query = lib.INTERNAL_QUERY_OPEN_EVENTS_BY_FEEDCODE
            ticket_kwargs = dict()

        self.execute(feedcodes_query)
        feedcodes = [x["feed.code"] for x in self.cur.fetchall()]
        if feedcodes:
            self.logger.info(
                "All feeds: " + ", ".join(["%r"] * len(feedcodes)) % tuple(feedcodes)
            )
        else:
            self.logger.info("Nothing to do.")
        for feedcode in feedcodes:
            self.logger.info("Handling feedcode {!r}.".format(feedcode))
            self.execute(events_query, (feedcode,))
            feeddata = []
            self.logger.info("Found %s events." % self.cur.rowcount)
            for row in self.cur:
                """
                First, we ignore None-data
                Second, we ignore raw
                Third, we convert everything to strings, e.g. datetime-objects
                """
                feeddata.append(
                    {
                        k: (str(v) if isinstance(v, datetime.datetime) else v)
                        for k, v in row.items()
                        if v is not None and k != "raw"
                    }
                )

            attachment = io.BytesIO()
            ziphandle = zipfile.ZipFile(
                attachment, mode="w", compression=zipfile.ZIP_DEFLATED
            )
            ziphandle.writestr(
                "events.json",
                json.dumps(feeddata, sort_keys=True, indent=4, separators=(",", ": ")),
            )
            ziphandle.close()
            attachment.seek(0)
            subject = "Reports of {} on {}".format(feedcode, time.strftime("%Y-%m-%d"))

            if self.dryrun:
                self.logger.info("Dry run: Skipping creation of report.")
                report_id = None
            else:
                report_id = self.rt.create_ticket(
                    Queue="Incident Reports",
                    Subject=subject,
                    Requestor=self.config["rt"]["incident_report_requestor"].format(
                        feedcode=feedcode
                    ),
                    **ticket_kwargs,
                )
                if report_id == -1:
                    self.logger.error(
                        "Could not create Incident ({}).".format(report_id)
                    )
                    return 1
                else:
                    self.logger.info("Created Report {}.".format(report_id))

            if self.dryrun:
                self.logger.info("Dry run: Skipping creation of attachment.")
            else:
                comment_id = self.rt.comment(
                    report_id, files=[("events.zip", attachment, "application/zip")]
                )
                if not comment_id:
                    self.logger.error("Could not correspond with file.")
                    return 1

            if not self.dryrun:
                self.executemany(
                    "UPDATE {events} SET rtir_report_id = %s WHERE id = %s",
                    [(report_id, row["id"]) for row in feeddata],
                    extend=False,
                )
                self.con.commit()
            self.logger.info("Linked events to report.")
        return 0


def main():
    controller = IntelMQCLIContoller()
    sys.exit(controller.run(sys.argv[1:]))


if __name__ == "__main__":
    main()



"""Tests for DISPParserBot

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import unittest
from copy import deepcopy
from datetime import datetime

import intelmq.lib.message as message
from dateutil.tz import UTC
from intelmq.lib.test import skip_internet

from intelmq_extensions.bots.parsers.disp.parser import DISPParserBot

from ....base import BotTestCase


class TestDISPParserBot(BotTestCase, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = DISPParserBot
        cls.sysconfig = {
            "default_fields": {
                "classification.taxonomy": "information-content-security",
                "classification.type": "data-leak",
                "classification.identifier": "leaked-credentials",
                "comment": "A comment",
            }
        }

    def _create_report_dict(self, credentials, id_: str = "id-1", **kwargs):
        incident_data = {
            "id": id_,
            "creationDate": 1693391274125,
            "publishedDate": 1693391312517,
            "title": "Credentials compromised by malware-1 botnet",
            "category": "Information discovery",
            "type": "Credentials",
            "detectionDate": 1693391031733,
            "validationDate": 1693391274125,
            "updateDate": 1693391446948,
            "riskLevel": "High",
            "tags": [
                "family: malware-1",
                "type: InfoStealer",
            ],
            "relatedAssets": ["example.at"],
        }
        default_credentials = {
            "malware": "malware-1",
            "date": "",
            "application": "Some App XXX",
            "url": "https://some.example.at/",
            "username": "my-user-1",
            "password": "pas*****",
        }
        credentials_data = []
        for credential in credentials:
            c_data = deepcopy(default_credentials)
            c_data.update(credential)
            credentials_data.append(c_data)
        evidence_data = {"credentials": credentials_data}

        raw = {"incident": incident_data, "evidences": evidence_data}
        data = {
            "feed.name": "Test Bot",
            "feed.accuracy": 100.0,
            "feed.code": "feed-code",
            "time.observation": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
        }
        data.update(kwargs)
        report = message.Report(data, harmonization=self.harmonization)
        report.add("raw", json.dumps(raw))
        return report.to_dict(with_type=True)

    def _create_event_dict(
        self,
        username: str = "my-user-1",
        password: str = "pas*****",
        fqdn: str = "some.example.at",
        url: str = "https://some.example.at/",
        urlpath: str = "/",
        full_url: str = "hxxps://some.example.at/",
        compromise_time: str = "",
        incident_data: dict = None,
        evidences_data: dict = None,
        compromise_time_full: str = None,
        ip: str = None,
        **kwargs,
    ):
        data = {
            "classification.identifier": "leaked-credentials",
            "classification.taxonomy": "information-content-security",
            "classification.type": "data-leak",
            "comment": "A comment",
            "event_description.text": "Credentials compromised by malware-1 botnet",
            "source.fqdn": fqdn,
            "source.urlpath": urlpath,
            "source.url": url,
            "source.ip": ip,
            "source.account": username,
            "extra.account": username,  # Intentionally copied
            "extra.password": password,
            "extra.compromise_time": compromise_time,
            "extra.compromise_time_full": compromise_time_full,
            "extra.application": "Some App XXX",
            "extra.full_url": full_url,
            "extra.feed_event_id": "id-1",
            "malware.name": "malware-1",
            "feed.code": "feed-code",
            "feed.name": "Test Bot",
            "feed.accuracy": 100.0,
            "extra.monitored_asset": "example.at",  # Domain we monitor in DISP
            "time.observation": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
        }
        data.update(kwargs)
        event = message.Event(data, harmonization=self.harmonization)
        event.add(
            "time.source",
            datetime.fromtimestamp(1693391274125 // 1000, tz=UTC).isoformat(),
        )
        event.add(
            "raw", json.dumps({"incident": incident_data, "evidences": evidences_data})
        )
        return event.to_dict(with_type=True)

    def test_parsing_report_default(self):
        self.input_message = self._create_report_dict(
            [
                {
                    "malware": "malware-1",
                    "date": "",
                    "application": "Some App XXX",
                    "url": "https://some.example.at/",
                    "username": "my-user-1",
                    "password": "pas*****",
                },
                {
                    "malware": "malware-1",
                    "date": "09.27.2021 16:17:18",
                    "application": "Some App XXX",
                    "url": "https://example.at/some/sub/page",
                    "username": "my-user-2",
                    "password": "lea*******",
                },
            ],
        )

        self.run_bot()

        self.assertMessageEqual(
            0,
            self._create_event_dict(username="my-user-1", password="pas*****"),
            compare_raw=False,
        )
        self.assertMessageEqual(
            1,
            self._create_event_dict(
                username="my-user-2",
                password="lea*******",
                fqdn="example.at",
                url="https://example.at/[REDACTED]",
                urlpath="/[REDACTED]",
                full_url="hxxps://example.at/some/sub/page",
                compromise_time="2021-09",
                compromise_time_full="09.27.2021 16:17:18",
            ),
            compare_raw=False,
        )

    def test_compromise_time_settings(self):
        report = self._create_report_dict([{"date": "09.27.2021 16:17:18"}])

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"compromise_time_format": "original"})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                compromise_time="09.27.2021 16:17:18",
                compromise_time_full="09.27.2021 16:17:18",
            ),
            compare_raw=False,
        )

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"compromise_time_format": ""})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                compromise_time="",
                compromise_time_full="09.27.2021 16:17:18",
            ),
            compare_raw=False,
        )

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"compromise_time_format": "%Y-%m-%d"})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                compromise_time="2021-09-27",
                compromise_time_full="09.27.2021 16:17:18",
            ),
            compare_raw=False,
        )

        self.input_message = self._create_report_dict([{"date": "invalid"}])
        self.run_bot(
            parameters={"compromise_time_format": "%Y-%m-%d"}, allowed_warning_count=1
        )
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                compromise_time="",
                compromise_time_full="invalid",
            ),
            compare_raw=False,
        )

    def test_redacting_url_settings(self):
        report = self._create_report_dict(
            [{"url": "https://some.example.at/my/sub?url=1"}]
        )

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"redact_url_path": True})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                fqdn="some.example.at",
                url="https://some.example.at/[REDACTED]",
                urlpath="/[REDACTED]",
                full_url="hxxps://some.example.at/my/sub?url=1",
            ),
            compare_raw=False,
        )

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"redact_url_path": False})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                fqdn="some.example.at",
                url="https://some.example.at/my/sub",
                urlpath="/my/sub",
                full_url="hxxps://some.example.at/my/sub?url=1",
            ),
            compare_raw=False,
        )

    @skip_internet()
    def test_resolve_ip_settings(self):
        report = self._create_report_dict([{"url": "https://cert.at/"}])

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"resolve_ip": True})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                fqdn="cert.at",
                ip="131.130.249.234",
                url="https://cert.at/",
                full_url="hxxps://cert.at/",
            ),
            compare_raw=False,
        )

        self.input_message = deepcopy(report)
        self.run_bot(parameters={"resolve_ip": False})
        self.assertMessageEqual(
            0,
            self._create_event_dict(
                fqdn="cert.at",
                ip="",
                url="https://cert.at/",
                full_url="hxxps://cert.at/",
            ),
            compare_raw=False,
        )



# -*- coding: utf-8 -*-
import base64
import os
import unittest

import intelmq.lib.test as test

from intelmq_extensions.bots.parsers.malwaredomains.parser import (
    MalwareDomainsParserBot,
)

with open(os.path.join(os.path.dirname(__file__), "domains.txt"), "rb") as fh:
    RAW = base64.b64encode(fh.read()).decode()

OUTPUT1 = {
    "__type": "Event",
    "classification.type": "phishing",
    "event_description.text": "phishing",
    "classification.identifier": "phishing",
    "raw": "CQlleGFtcGxlLmNvbQlwaGlzaGluZwlvcGVucGhpc2guY29tCTIwMTYwNTI3CTIwMTYwMTA4",
    "source.fqdn": "example.com",
    "time.source": "2016-05-27T00:00:00+00:00",
}
OUTPUT2 = {
    "__type": "Event",
    "classification.type": "phishing",
    "event_description.text": "phishing",
    "classification.identifier": "phishing",
    "raw": "CQlleGFtcGxlLmludmFsaWQJcGhpc2hpbmcJb3BlbnBoaXNoLmNvbQkyMDE2MDUyNwkyMDE2MDEwOA==",
    "source.fqdn": "example.invalid",
    "time.source": "2016-05-27T00:00:00+00:00",
}
OUTPUT3 = {
    "__type": "Event",
    "classification.type": "c2-server",
    "event_description.text": "C&C",
    "classification.identifier": "C&C",
    "raw": "CQlleGFtcGxlLm5ldAlDJkMJc291cmNlLmV4YW1wbGUuY29tCTIwMTcxMjAxCTIwMTYwNzE5CTIwMTYwMzEw",
    "source.fqdn": "example.net",
    "time.source": "2017-12-01T00:00:00+00:00",
}


class TestMalwareDomainsParserBot(test.BotTestCase, unittest.TestCase):
    """
    A TestCase for MalwareDomainsParserBot.
    """

    @classmethod
    def set_bot(cls):
        cls.bot_reference = MalwareDomainsParserBot
        cls.default_input_message = {"__type": "Report", "raw": RAW}

    def test_event(self):
        self.run_bot()
        self.assertMessageEqual(0, OUTPUT1)
        self.assertMessageEqual(1, OUTPUT2)
        self.assertMessageEqual(2, OUTPUT3)


if __name__ == "__main__":  # pragma: no cover
    unittest.main()



"""Tests for BlackKite Collector

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import unittest
from unittest import mock

import intelmq.lib.message as message
import requests

from intelmq_extensions.bots.collectors.blackkite.collector import BlackKiteCollectorBot

from ....base import BotTestCase
from .base import BlackKite_APIMockMixIn


class TestBlackKiteCollectorBot(BotTestCase, BlackKite_APIMockMixIn, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = BlackKiteCollectorBot
        cls.sysconfig = {
            "url": "https://blackkite.example.at/v1",
            "client_id": "client1",
            "client_secret": "secret1",
            "code": "feed-code",
            "categories": {"PATCH": {}},
        }

    def catch_http_session(self):
        session = requests.Session()
        session_mock = mock.patch(
            "intelmq_extensions.bots.collectors.blackkite.collector.create_request_session",
            return_value=session,
        )
        session_mock.start()
        self.addCleanup(session_mock.stop)
        return session

    def setUp(self) -> None:
        super().setUp()
        session = self.catch_http_session()
        self.setup_api_mock(session=session)

    def test_static_check_validates_filters(self):
        correct_params = {
            "severities": ["Info", "Low", "Medium", "High", "Critical"],
            "statuses": [
                "Active",
                "FalsePositive",
                "Suppressed",
                "Acknowledged",
                "Deleted",
            ],
            "outputs": ["Info", "Passed", "Warning", "Failed"],
            "categories": {
                "PATCH": {
                    "statuses": ["Active"],
                    "severities": ["Critical"],
                    "include": ["PATCH-001"],
                },
                "APPSEC": {},
                "DNS": {"outputs": ["Failed"]},
                "LEAK": None,
            },
        }
        self.assertIsNone(self.bot_reference.check(correct_params))

    def test_static_check_invalid_params(self):
        invalid_params = {
            "severities": ["Some", "Other"],
            "statuses": ["NonActive"],
            "outputs": ["Other"],
            "categories": {
                "PATCH": {
                    "include": ["PATCH-001"],
                    "exclude": ["PATCH-002"],
                },
                "DNS": {
                    "statuses": ["NonActive"],
                },
                "FRADOM": {
                    "outputs": ["Failed"],
                },
                "non-existing": {},
                "SMTP": {"not-existing-key": []},
            },
        }
        results = self.bot_reference.check(invalid_params)
        self.assertEqual(8, len([r for r in results if r[0] == "error"]))

    def _create_report_dict(self, company_data: dict, finding_data: dict, **kwargs):
        report = message.Report(kwargs, harmonization=self.harmonization)
        data = {"company": company_data, "finding": finding_data}
        report.add("feed.name", "Test Bot")
        report.add("feed.accuracy", 100.0)
        report.add("feed.code", "feed-code")
        report.add("raw", json.dumps(data))
        return report.to_dict(with_type=True)

    def _mock_companies(self, number: int = 2):
        self.mock_request(
            "companies",
            json=[{"CompanyId": id_ + 1} for id_ in range(number)],
            headers={"X-Total-Items": str(number)},
        )

    def test_get_data_for_all_companies(self):
        self._mock_companies()
        self.mock_request(
            "companies/1/findings/patchmanagement?" "status=Active&severity=Critical",
            json=[{"FindingId": 1}, {"FindingId": 2}],
        )
        self.mock_request(
            "companies/2/findings/patchmanagement?&status=Active&severity=Critical",
            json=[{"FindingId": 3}],
        )

        self.run_bot()

        self.assertOutputQueueLen(3)
        self.assertMessageEqual(
            0, self._create_report_dict({"CompanyId": 1}, {"FindingId": 1})
        )
        self.assertMessageEqual(
            1, self._create_report_dict({"CompanyId": 1}, {"FindingId": 2})
        )
        self.assertMessageEqual(
            2, self._create_report_dict({"CompanyId": 2}, {"FindingId": 3})
        )

    def test_single_exception_doesnt_stop_processing(self):
        self._mock_companies()
        self.mock_request(
            "companies/1/findings/patchmanagement?status=Active&severity=Critical",
            status_code=500,
        )
        self.mock_request(
            "companies/1/findings/applicationsecurity?status=Active&severity=Critical",
            json=[{"FindingId": 1}],
        )
        self.mock_request(
            "companies/2/findings/patchmanagement?status=Active&severity=Critical",
            json=[{"FindingId": 2}],
        )
        self.mock_request(
            "companies/2/findings/applicationsecurity?status=Active&severity=Critical",
            json=[{"FindingId": 3}],
        )

        self.run_bot(
            parameters={**self.sysconfig, "categories": {"PATCH": {}, "APPSEC": {}}},
            allowed_error_count=2,  # log from collector + from client
        )

        self.assertOutputQueueLen(3)
        self.assertMessageEqual(
            0, self._create_report_dict({"CompanyId": 1}, {"FindingId": 1})
        )
        self.assertMessageEqual(
            1, self._create_report_dict({"CompanyId": 2}, {"FindingId": 2})
        )
        self.assertMessageEqual(
            2, self._create_report_dict({"CompanyId": 2}, {"FindingId": 3})
        )

    def test_overriding_filters(self):
        self._mock_companies(1)
        self.mock_request(
            "companies/1/findings/patchmanagement?status=Acknowledged&severity=Critical",
            json=[{"FindingId": 1}],
        )
        self.mock_request(
            "companies/1/findings/applicationsecurity?status=Active&severity=Low,Critical",
            headers={"X-Total-Items": "1"},
            json=[{"FindingId": 2}],
        )

        self.run_bot(
            parameters={
                **self.sysconfig,
                "severities": ["Critical"],
                "statuses": ["Active"],
                "categories": {
                    "PATCH": {"statuses": ["Acknowledged"]},
                    "APPSEC": {"severities": ["Low", "Critical"]},
                },
            }
        )

        self.assertOutputQueueLen(2)
        self.assertMessageEqual(
            0, self._create_report_dict({"CompanyId": 1}, {"FindingId": 1})
        )
        self.assertMessageEqual(
            1, self._create_report_dict({"CompanyId": 1}, {"FindingId": 2})
        )

    def test_category_with_no_settings(self):
        self._mock_companies(1)
        self.mock_request(
            "companies/1/findings/patchmanagement?status=Active&severity=Critical",
            json=[{"FindingId": 1}],
        )

        self.run_bot(
            parameters={
                **self.sysconfig,
                "categories": {
                    "PATCH": None,
                },
            }
        )

        self.assertOutputQueueLen(1)
        self.assertMessageEqual(
            0, self._create_report_dict({"CompanyId": 1}, {"FindingId": 1})
        )

    def test_include_and_exclude_findings(self):
        self._mock_companies(1)
        self.mock_request(
            "companies/1/findings/patchmanagement",
            json=[
                {"ControlId": "PATCH-001"},
                {"ControlId": "PATCH-002"},
                {"ControlId": "PATCH-003"},
            ],
        )
        self.mock_request(
            "companies/1/findings/applicationsecurity",
            headers={"X-Total-Items": "1"},
            json=[
                {"ControlId": "APPSEC-001"},
                {"ControlId": "APPSEC-002"},
                {"ControlId": "APPSEC-003"},
            ],
        )

        self.run_bot(
            parameters={
                **self.sysconfig,
                "categories": {
                    "PATCH": {"include": ["PATCH-001", "PATCH-003"]},
                    "APPSEC": {"exclude": ["APPSEC-003"]},
                },
            }
        )

        self.assertOutputQueueLen(4)
        self.assertMessageEqual(
            0, self._create_report_dict({"CompanyId": 1}, {"ControlId": "PATCH-001"})
        )
        self.assertMessageEqual(
            1, self._create_report_dict({"CompanyId": 1}, {"ControlId": "PATCH-003"})
        )
        self.assertMessageEqual(
            2, self._create_report_dict({"CompanyId": 1}, {"ControlId": "APPSEC-001"})
        )
        self.assertMessageEqual(
            3, self._create_report_dict({"CompanyId": 1}, {"ControlId": "APPSEC-002"})
        )

    def test_acknowledge_findings(self):
        self._mock_companies(1)
        self.mock_request(
            "companies/1/findings/credentialmanagement",
            json=[
                {"ControlId": "LEAK-001", "FindingId": 1},
                {"ControlId": "LEAK-002", "FindingId": 2},
            ],
            headers={"X-Total-Items": "2"},
        )
        self.mock_request(
            "companies/1/findings/1", json={"Status": "Acknowledged"}, method="patch"
        )
        self.mock_request(
            "companies/1/findings/2", json={"Status": "Acknowledged"}, method="patch"
        )

        self.run_bot(
            parameters={**self.sysconfig, "categories": {"LEAK": {"acknowledge": True}}}
        )

        self.assertEqual(2 + 2, self.requests.call_count)



# -*- coding: utf-8 -*-
"""
Utilities for intelmqcli.

Static data (queries)
"""
import argparse
import copy
import datetime
import json
import os
import subprocess
import sys

import intelmq
import intelmq.lib.utils as utils
import pkg_resources
import psycopg2
import psycopg2.extras
import rt
from intelmq import CONFIG_DIR

INTELMQCLI_CONF_FILE = os.path.join(CONFIG_DIR, "intelmqcli.conf")


__all__ = [
    "BASE_WHERE",
    "CSV_FIELDS",
    "EPILOG",
    "QUERY_DISTINCT_CONTACTS_BY_INCIDENT",
    "QUERY_EVENTS_BY_ASCONTACT_INCIDENT",
    "QUERY_FEED_NAMES",
    "QUERY_GET_TEXT",
    "QUERY_IDENTIFIER_NAMES",
    "QUERY_INSERT_CONTACT",
    "QUERY_OPEN_EVENTS_BY_FEEDCODE",
    "QUERY_HALF_PROC_INCIDENTS",
    "QUERY_OPEN_EVENT_IDS_BY_TAXONOMY",
    "QUERY_OPEN_EVENT_REPORTS_BY_TAXONOMY",
    "QUERY_OPEN_FEEDCODES",
    "QUERY_OPEN_TAXONOMIES",
    "QUERY_TAXONOMY_NAMES",
    "QUERY_TEXT_NAMES",
    "QUERY_TYPE_NAMES",
    "QUERY_UPDATE_CONTACT",
    "USAGE",
    "getTerminalHeight",
    "IntelMQCLIContollerTemplate",
]

EPILOG = """
Searches for all unprocessed incidents. Incidents will be filtered by country
code and the TLD of a domain according to configuration.
The search can be restricted to one source feed.

After the start, intelmqcli will immediately connect to RT with the given
credentials. The incidents will be shown grouped by the contact address if
known or the ASN otherwise.

You have 3 options here:
* Select one group by giving the id (number in first column) and show the email
and all events in detail
* Automatic sending of all incidents with 'a'
* Quit with 'q'

For the detailed view, the recipient, the subject and the mail text will be
shown, and below the technical data as csv. If the terminal is not big enough,
the data will not be shown in full. In this case, you can press 't' for the
table mode. less will be opened with the full text and data, whereas the data
will be formated as table, which is much easier to read and interpret.
The requestor (recipient of the mail) can be changed manually by pressing 'r'
and in the following prompt the address is asked. After sending, you can
optionally save the (new) address to the database linked to the ASNs.
If you are ready to submit the incidents to RT and send the mails out, press
's'.
'b' for back jumps to the incident overview and 'q' quits.

Exit codes:
 0 if no errors happend or only errors which could be handled. Check the
   output if recoverable errors happened.
 1 if unrecoverable errors happened
 2 if user input or configuration is faulty
"""
USAGE = """
    intelmqcli
    intelmqcli --dry-run
    intelmqcli --verbose
    intelmqcli --batch
    intelmqcli --quiet
    intelmqcli --compress-csv
    intelmqcli --list-feeds
    intelmqcli --list-identifiers
    intelmqcli --list-taxonomies
    intelmqcli --taxonomy='taxonomy'
    intelmqcli --type='type'
    intelmqcli --identifier='identifier'
    intelmqcli --list-types
    intelmqcli --list-texts
    intelmqcli --text='boilerplate name'
    intelmqcli --feed='feedcode' """

SUBJECT = {
    "abusive-content": "Abusive content (spam, ...)",
    "malicious code": "Malicious code (malware, botnet, ...)",
    "malicious-code": "Malicious code (malware, botnet, ...)",
    "information-gathering": "Information Gathering (scanning, ...)",
    "intrusion-attempts": "Intrusion Attempt",
    "intrusions": "Network intrusion",
    "availability": "Availability (DDOS, ...)",
    "information-content-security": "Information Content Security (dropzone,...)",
    "fraud": "Fraud",
    "vulnerable": "Vulnerable device",
    "other": "Other",
    "test": "Test",
}

QUERY_FEED_NAMES = 'SELECT DISTINCT "feed.code" from {events}'

QUERY_IDENTIFIER_NAMES = 'SELECT DISTINCT "classification.identifier" from {events}'

QUERY_TAXONOMY_NAMES = 'SELECT DISTINCT "classification.taxonomy" from {events}'

QUERY_TYPE_NAMES = 'SELECT DISTINCT "classification.type" from {events}'

QUERY_TEXT_NAMES = 'SELECT DISTINCT "key" from {boilerplates}'

""" This is the list of fields (and their respective order) which we intend to
send out.  This is based on the order and fields of shadowserver.

Shadowserver format:
    timestamp,"ip","protocol","port","hostname","packets","size","asn","geo","region","city","naics","sic","sector"
"""
CSV_FIELDS = [
    "time.source",
    "source.ip",
    "protocol.transport",
    "source.port",
    "protocol.application",
    "source.fqdn",
    "source.local_hostname",
    "source.local_ip",
    "source.url",
    "source.asn",
    "source.geolocation.cc",
    "source.geolocation.city",
    "classification.taxonomy",
    "classification.type",
    "classification.identifier",
    "destination.ip",
    "destination.port",
    "destination.fqdn",
    "destination.url",
    "feed",
    "event_description.text",
    "event_description.url",
    "malware.name",
    "extra",
    "comment",
    "additional_field_freetext",
    "feed.documentation",
    "version: 1.2",
]

QUERY_UPDATE_CONTACT = """
UPDATE as_contacts SET
    contacts = %s
WHERE
    asnum = %s
"""

QUERY_INSERT_CONTACT = """
INSERT INTO as_contacts (
    asnum, contacts, comment, unreliable
) VALUES (
    %s, %s, %s, FALSE
)
"""

QUERY_GET_TEXT = """
SELECT
    body
FROM {texttab}
WHERE
    key = %s
"""

_BASE_FILTERS = """
"notify" = TRUE AND
"time.source" >= now() - interval %s AND
"sent_at" IS NULL AND
"feed.code" IS NOT NULL AND
"classification.taxonomy" IS NOT NULL AND
(UPPER("source.geolocation.cc") = %s OR lower("source.fqdn") similar to %s
OR lower("source.reverse_dns") similar to %s {asset})
"""
BASE_WHERE = (
    """
"source.abuse_contact" IS NOT NULL AND
"""
    + _BASE_FILTERS
)
_MONITORED_ASSETS_FILTER = """
OR "extra" ->> 'monitored_asset' != ''
"""
# PART 1: CREATE REPORTS
_OPEN_FEEDCODES = """
SELECT
    DISTINCT "feed.code"
FROM "{events}"
WHERE
    "rtir_report_id" IS NULL AND
"""
QUERY_OPEN_FEEDCODES = _OPEN_FEEDCODES + BASE_WHERE
_OPEN_EVENTS = """
SELECT *
FROM "{events}"
WHERE
    "feed.code" = %s AND
    "rtir_report_id" IS NULL AND
"""
QUERY_OPEN_EVENTS_BY_FEEDCODE = _OPEN_EVENTS + BASE_WHERE
# For internal-only notifications, ignore abuse data
INTERNAL_QUERY_OPEN_FEEDCODES = _OPEN_FEEDCODES + _BASE_FILTERS
INTERNAL_QUERY_OPEN_EVENTS_BY_FEEDCODE = _OPEN_EVENTS + _BASE_FILTERS
# PART 2: INCIDENTS
QUERY_OPEN_TAXONOMIES = (
    """
SELECT
    DISTINCT "classification.taxonomy"
FROM "{events}"
WHERE
    "rtir_report_id" IS NOT NULL AND
    "rtir_incident_id" IS NULL AND
"""
    + BASE_WHERE
)
QUERY_OPEN_EVENT_REPORTS_BY_TAXONOMY = (
    """
SELECT
    DISTINCT "rtir_report_id"
FROM "{events}"
WHERE
    "rtir_report_id" IS NOT NULL AND
    "rtir_incident_id" IS NULL AND
    "classification.taxonomy" = %s AND
"""
    + BASE_WHERE
)
QUERY_OPEN_EVENT_IDS_BY_TAXONOMY = (
    """
SELECT
    "id"
FROM "{events}"
WHERE
    "rtir_report_id" IS NOT NULL AND
    "rtir_incident_id" IS NULL AND
    "classification.taxonomy" = %s AND
"""
    + BASE_WHERE
)
QUERY_HALF_PROC_INCIDENTS = (
    """
SELECT
    DISTINCT "rtir_incident_id",
    "classification.taxonomy"
FROM "{events}"
WHERE
    "rtir_report_id" IS NOT NULL AND
    "rtir_incident_id" IS NOT NULL AND
    rtir_investigation_id IS NULL AND
"""
    + BASE_WHERE
)
# PART 3: INVESTIGATIONS
QUERY_DISTINCT_CONTACTS_BY_INCIDENT = (
    """
SELECT
DISTINCT "source.abuse_contact"
FROM {events}
WHERE
    rtir_report_id IS NOT NULL AND
    rtir_incident_id = %s AND
    rtir_investigation_id IS NULL AND
"""
    + BASE_WHERE
)
DRY_QUERY_DISTINCT_CONTACTS_BY_TAXONOMY = (
    """
SELECT
DISTINCT "source.abuse_contact"
FROM {events}
WHERE
    rtir_report_id IS NOT NULL AND
    "rtir_incident_id" IS NULL AND
    rtir_investigation_id IS NULL AND
    "classification.taxonomy" = %s AND
"""
    + BASE_WHERE
)
QUERY_EVENTS_BY_ASCONTACT_INCIDENT = (
    """
SELECT
    to_char("time.source",
            'YYYY-MM-DD"T"HH24:MI:SSOF') as "time.source",
    id,
    "feed.code" as feed,
    "source.ip",
    "source.port",
    "source.url",
    "source.asn",
    "source.geolocation.cc",
    "source.geolocation.city",
    "source.fqdn",
    "source.local_hostname",
    "source.local_ip",
    "classification.identifier",
    "classification.taxonomy",
    "classification.type",
    "comment",
    "destination.ip",
    "destination.port",
    "destination.fqdn",
    "destination.url",
    "event_description.text",
    "event_description.url",
    "shareable_extra_info" AS "extra",
    "feed.documentation",
    "malware.name",
    "protocol.application",
    "protocol.transport"
FROM {v_events_filtered}
WHERE
    rtir_report_id IS NOT NULL AND
    rtir_incident_id = %s AND
    rtir_investigation_id IS NULL AND
    "source.abuse_contact" = %s AND
"""
    + BASE_WHERE
)
DRY_QUERY_EVENTS_BY_ASCONTACT_TAXONOMY = (
    QUERY_EVENTS_BY_ASCONTACT_INCIDENT[
        : QUERY_EVENTS_BY_ASCONTACT_INCIDENT.find("WHERE") + 6
    ]
    + """
    rtir_report_id IS NOT NULL AND
    rtir_investigation_id IS NULL AND
    "classification.taxonomy" = %s AND
    "source.abuse_contact" = %s AND
"""
    + BASE_WHERE
)

DEFAULT_TABLES = {
    "events": "events",
    "v_events_filtered": "v_events_filtered",
    "boilerplates": "boilerplates",
}


def getTerminalHeight():
    try:
        return int(subprocess.check_output(["stty", "size"]).strip().split()[0])
    except Exception:
        return 80  # If running in


class IntelMQCLIContollerTemplate:
    additional_where = ""
    usage = ""
    epilog = ""
    additional_params = ()
    dryrun = False
    quiet = False

    def __init__(self, overridden_config: dict = None):
        self._asset_filter = ""
        self.overridden_config = overridden_config

        usage_configuration = (
            "\n\nThe configuration can be found by default at %r."
            % INTELMQCLI_CONF_FILE
        )

        self.parser = argparse.ArgumentParser(
            prog=self.appname,
            usage=self.usage + usage_configuration,
            epilog=self.epilog,
            formatter_class=argparse.RawDescriptionHelpFormatter,
        )
        VERSION = pkg_resources.get_distribution("intelmq").version
        self.parser.add_argument("--version", action="version", version=VERSION)
        self.parser.add_argument(
            "-v", "--verbose", action="store_true", help="Print verbose messages."
        )

        self.parser.add_argument(
            "--config", help="Config file path", default=INTELMQCLI_CONF_FILE
        )

        self.parser.add_argument(
            "-f",
            "--feed",
            nargs="+",
            help="Show only incidents reported by one of the given feeds.",
        )
        self.parser.add_argument(
            "--skip-feed",
            nargs="+",
            help="Skip incidents reported by one of the given feeds.",
        )
        self.parser.add_argument(
            "--taxonomy", nargs="+", help="Select only events with given taxonomy."
        )
        self.parser.add_argument(
            "--skip-taxonomy", nargs="+", help="Skip events with given taxonomy."
        )
        self.parser.add_argument(
            "-a",
            "--asn",
            type=int,
            nargs="+",
            help="Specify one or more AS numbers (integers) to process.",
        )
        self.parser.add_argument(
            "--skip-asn",
            type=int,
            nargs="+",
            help="Specify one or more AS numbers (integers) to skip.",
        )
        self.parser.add_argument(
            "--type",
            nargs="+",
            help="Specify one or more classifications types to process.",
        )
        self.parser.add_argument(
            "--skip-type",
            nargs="+",
            help="Specify one or more classifications types to skip.",
        )
        self.parser.add_argument(
            "--identifier",
            nargs="+",
            help="Specify one or more classifications identifiers to process.",
        )
        self.parser.add_argument(
            "--skip-identifier",
            nargs="+",
            help="Specify one or more classifications identifiers to skip.",
        )

        self.parser.add_argument(
            "-b",
            "--batch",
            action="store_true",
            help='Run in batch mode (defaults to "yes" to all).',
        )
        self.parser.add_argument(
            "-q",
            "--quiet",
            action="store_true",
            help="Do not output anything, except for error messages."
            " Useful in combination with --batch.",
        )
        self.parser.add_argument(
            "-n",
            "--dry-run",
            action="store_true",
            help="Do not store anything or change anything. Just simulate.",
        )

        self.parser.add_argument(
            "--time-interval",
            nargs="+",
            default="4 days",
            help="time interval, parseable by postgres." 'defaults to "4 days".',
        )
        self.parser.add_argument(
            "--ton",
            "--time-observation-newer-than",
            nargs=1,
            help="Select only events with 'time.observation' newer than the "
            "given ISO-formatted datetime.",
        )
        self.parser.add_argument(
            "--monitored-assets",
            action="store_true",
            help="Include tickets that has extra.monitored_asset regardless of geo-filters",
        )
        self.parser.add_argument(
            "--stdout-only", action="store_true", help="Log only to stdout"
        )

    def setup(self, args: list):
        self.args = self.parser.parse_args(args)

        self.config = self._load_config()

        if self.args.verbose:
            self.verbose = True
        if self.args.dry_run:
            self.dryrun = True
        if self.args.batch:
            self.batch = True
        if self.args.quiet:
            self.quiet = True
        self.time_interval = "".join(self.args.time_interval)

        if self.quiet:
            stream = None
        else:
            stream = sys.stderr

        self.logger = utils.log(
            "intelmqcli",
            syslog=(
                None if self.args.stdout_only else self.config.get("syslog", "/dev/log")
            ),
            log_level=self.config["log_level"].upper(),
            stream=stream,
            log_format_stream="%(message)s",
            log_path=(
                None
                if self.args.stdout_only
                else self.config.get("log_path", intelmq.DEFAULT_LOGGING_PATH)
            ),
        )
        self.logger.info(
            "Started %r at %s.", " ".join(sys.argv), datetime.datetime.now().isoformat()
        )

        if self.args.feed:
            self.additional_where += """ AND "feed.code" = ANY(%s::VARCHAR[]) """
            self.additional_params += ("{" + ",".join(self.args.feed) + "}",)
        if self.args.skip_feed:
            self.additional_where += """ AND "feed.code" != ANY(%s::VARCHAR[]) """
            self.additional_params += ("{" + ",".join(self.args.skip_feed) + "}",)
        if self.args.asn:
            self.additional_where += """ AND "source.asn" = ANY(%s::INT[]) """
            self.additional_params += ("{" + ",".join(map(str, self.args.asn)) + "}",)
        if self.args.skip_asn:
            self.additional_where += """ AND "source.asn" != ANY(%s::INT[]) """
            self.additional_params += (
                "{" + ",".join(map(str, self.args.skip_asn)) + "}",
            )
        if self.args.taxonomy:
            self.additional_where += (
                """ AND "classification.taxonomy" = ANY(%s::VARCHAR[]) """
            )
            self.additional_params += ("{" + ",".join(self.args.taxonomy) + "}",)
        if self.args.skip_taxonomy:
            self.additional_where += (
                """ AND "classification.taxonomy" != ANY(%s::VARCHAR[]) """
            )
            self.additional_params += ("{" + ",".join(self.args.skip_taxonomy) + "}",)
        if self.args.type:
            self.additional_where += (
                """ AND "classification.type" = ANY(%s::VARCHAR[]) """
            )
            self.additional_params += ("{" + ",".join(self.args.type) + "}",)
        if self.args.skip_type:
            self.additional_where += (
                """ AND "classification.type" != ANY(%s::VARCHAR[]) """
            )
            self.additional_params += ("{" + ",".join(self.args.skip_type) + "}",)
        if self.args.identifier:
            self.additional_where += (
                """ AND "classification.identifier" = ANY(%s::VARCHAR[]) """
            )
            self.additional_params += ("{" + ",".join(self.args.identifier) + "}",)
        if self.args.skip_identifier:
            self.additional_where += (
                """ AND "classification.identifier" != ALL(%s::VARCHAR[]) """
            )
            self.additional_params += ("{" + ",".join(self.args.skip_identifier) + "}",)
        if self.args.ton:
            self.additional_where += """ AND "time.observation" >= %s """
            self.additional_params += (self.args.ton[0],)
        if self.args.monitored_assets:
            self._asset_filter = _MONITORED_ASSETS_FILTER

        if self.config.get("constituency"):
            conditions = []
            if self.config["constituency"].get("default"):
                conditions.append(""" "constituency" is null """)

            if key := self.config["constituency"].get("key"):
                conditions.append(""" "constituency" = %s """)
                self.additional_params += (key,)

            if conditions:
                self.additional_where += f""" AND ({' OR '.join(conditions)}) """
                self.logger.debug("Initializing with constituency: %s", conditions)

        self.rt = rt.Rt(
            self.config["rt"]["uri"],
            self.config["rt"]["user"],
            self.config["rt"]["password"],
        )

    def _load_config(self):
        if self.overridden_config:
            return copy.deepcopy(self.overridden_config)

        with open(self.args.config) as conf_handle:
            return json.load(conf_handle)

    def connect_database(self):
        self.con = psycopg2.connect(
            database=self.config["database"]["database"],
            user=self.config["database"]["user"],
            password=self.config["database"]["password"],
            host=self.config["database"]["host"],
            port=self.config["database"]["port"],
            sslmode=self.config["database"]["sslmode"],
        )
        self.con.autocommit = False  # Starts transaction in the beginning
        self.cur = self.con.cursor(cursor_factory=psycopg2.extras.RealDictCursor)

    def execute(self, query, parameters=(), extend=True):
        """
        Passes query to database.

        Parameters:
            extend:
                If True, the parameters for BASE_WHERE are added (time interval,
                country code, FQDN from config, and additional parameters)
                If False, parameters are used as given.
        """
        if extend:
            query = query + self.additional_where
            parameters = (
                parameters
                + (
                    self.time_interval,
                    self.config["filter"]["cc"],
                    "%%.(%s)" % self.config["filter"]["fqdn"],
                    "%%.(%s)" % self.config["filter"]["fqdn"],
                )
                + self.additional_params
            )
        query = self._format_query(query)
        self.logger.debug(self.cur.mogrify(query, parameters))
        if not self.dryrun or query.strip().upper().startswith("SELECT"):
            self.cur.execute(query, parameters)

    def _format_query(self, query: str):
        return query.format(
            **self.config.get("tables", DEFAULT_TABLES), asset=self._asset_filter
        )

    def executemany(self, query, parameters=(), extend=True):
        """Passes query to database."""
        if extend:
            query = query + self.additional_where
            parameters = [
                param + (self.time_interval,) + self.additional_params
                for param in parameters
            ]
        query = self._format_query(query)
        if (
            self.config["log_level"].upper() == "DEBUG"
        ):  # on other log levels we can skip the iteration
            for param in parameters:
                self.logger.debug(self.cur.mogrify(query, param))
            if not parameters:
                self.logger.debug(self.cur.mogrify(query))
        if not self.dryrun or query.strip().upper().startswith(
            "SELECT"
        ):  # no update in dry run
            self.cur.executemany(query, parameters)



import contextlib
import copy
import io
import json
import os
import pathlib
import shutil
import sys
import tempfile
import unittest
from unittest import mock

import intelmq.lib.message as message
import intelmq.lib.test as test
import intelmq.lib.utils as utils
import pkg_resources
import psycopg2
from intelmq import CONFIG_DIR, RUNTIME_CONF_FILE
from intelmq.lib.harmonization import DateTime
from psycopg2 import sql
from rt import Rt

from intelmq_extensions.cli import lib

from ..cli.utils import merge_harmonization

ADDITIONAL_HARMONIZATION = ["contrib/constituency.harmonization.part.json"]
# DB_FIELDS = []  # selection of fields to save in DB when using test entries creation

POSTGRES_CONFIG = {
    "host": os.getenv("INTELMQ_TEST_DATABASE_HOST", "localhost"),
    "port": os.getenv("INTELMQ_TEST_DATABASE_PORT", 5432),
    "database": "intelmq",
    "user": "intelmq",
    "password": "intelmq",
    "sslmode": "allow",
    "text_table": "boilerplates_tests",  # TODO: move to tables config
}


def mocked_config_from_extensions(
    bot_id="test-bot", sysconfig={}, group=None, module=None
):
    """The only one difference with original is the package used to load resources"""

    def mocked(conf_file):
        if conf_file == RUNTIME_CONF_FILE:
            return {
                bot_id: {
                    "description": "Instance of a bot for automated unit tests.",
                    "group": group,
                    "module": module,
                    "name": "Test Bot",
                    "parameters": sysconfig,
                }
            }
        elif conf_file.startswith(CONFIG_DIR):
            confname = os.path.join("etc/", os.path.split(conf_file)[-1])
            fname = pkg_resources.resource_filename("intelmq_extensions", confname)
            with open(fname) as fpconfig:
                return json.load(fpconfig)
        else:
            return utils.load_configuration(conf_file)

    return mocked


test.mocked_config = mocked_config_from_extensions


def load_harmonization():
    harmonization = pkg_resources.resource_filename(
        "intelmq_extensions", "etc/harmonization.conf"
    )
    extensions = []
    for file in ADDITIONAL_HARMONIZATION:
        with open(pathlib.Path(__file__).parent.parent.parent / file) as f:
            extensions.append(json.load(f))
    return merge_harmonization(extensions, harmonization)


class TestCaseMixin:
    harmonization = load_harmonization()

    TEST_EVENTS_TABLE = "tests"
    TEST_TEXT_TABLE = "boilerplates_tests"

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self._tmp_dir = None

    @property
    def tmp_dir(self):
        if not self._tmp_dir:
            self._tmp_dir = tempfile.mkdtemp(prefix="intelmq_extensions_test_")
        return self._tmp_dir

    def tearDown(self):
        if self._tmp_dir:
            shutil.rmtree(self._tmp_dir)

        super().tearDown()

    def new_event(self):
        return message.Event(harmonization=self.harmonization)

    def get_mocked_logger(self, logger):
        def log(name, *args, **kwargs):
            logger.handlers = self.logger_handlers_backup
            return logger

        return log

    @staticmethod
    def connect_database(config: dict):
        connection = psycopg2.connect(
            database=config["database"],
            user=config["user"],
            password=config["password"],
            host=config["host"],
            port=config["port"],
            sslmode=config["sslmode"],
        )
        connection.autocommit = True
        return connection

    def db_add_event(
        self,
        data: dict,
        cc: str = "AT",
        fqdn: str = "test.at",
        notify: bool = True,
        taxonomy: str = "test",
        abuse_contact: str = "tes@test.at",
        extra: dict = None,
    ) -> int:
        """Adds event to the test DB. Some common fields required by CLI have default values

        data: any dict representing an event
        """
        extra = extra or dict()
        event = message.Event(harmonization=self.harmonization)
        event.add("source.geolocation.cc", cc)
        event.add("source.fqdn", fqdn)
        event.add("classification.taxonomy", taxonomy)
        event.add("source.abuse_contact", abuse_contact)
        event.add("notify", notify)
        event.add("time.source", DateTime.generate_datetime_now())

        event.update(data)

        keys = list(event.keys()) + ["extra"]
        values = list(event.values()) + [json.dumps(extra)]
        inserted_id = self._db_insert(self.TEST_EVENTS_TABLE, keys, values)
        self._clear_db_events.append(inserted_id)

        for key, value in extra.items():
            event.add(f"extra.{key}", value)

        return inserted_id

    def _db_insert(self, table: str, keys, values, return_key: str = "id"):
        query = sql.SQL("INSERT INTO {} ({}) VALUES ({}) RETURNING {};").format(
            sql.Identifier(table),
            sql.SQL(",").join(map(sql.Identifier, keys)),
            sql.SQL(",").join([sql.SQL("%s")] * len(keys)),
            sql.Identifier(return_key),
        )
        self.cur.execute(query, list(values))
        inserted_id = self.cur.fetchone()[return_key]

        return inserted_id

    def db_get_event(self, event_id: int):
        query = sql.SQL("SELECT * from {} WHERE id=%s").format(
            sql.Identifier(self.TEST_EVENTS_TABLE)
        )
        self.cur.execute(query, (event_id,))
        return self.cur.fetchone()

    def db_delete(self, table, ids: list, id_field: str = "id"):
        query = sql.SQL("DELETE FROM {} WHERE {} = ANY(%s);").format(
            sql.Identifier(table),
            sql.Identifier(id_field),
        )
        self.cur.execute(query, (ids,))

    def add_boilerplate(self, key: str, body: str):
        self._db_insert(
            self.TEST_TEXT_TABLE, ["key", "body"], [key, body], return_key="key"
        )
        self.addCleanup(
            lambda: self.db_delete(self.TEST_TEXT_TABLE, [key], id_field="key")
        )

    def assertInLogs(self, msg_part):
        self.log_stream: io.StringIO
        self.log_stream.seek(0)
        self.assertTrue(
            any(msg_part in line for line in self.log_stream),
            f"'{msg_part}' not found in captured logs",
        )


class BotTestCase(TestCaseMixin, test.BotTestCase):
    """Provides test class with additional changes required for extension bots"""


class CLITestCase(TestCaseMixin, unittest.TestCase):
    """Provides helpers needed to test the intelmqcli"""

    CLI_CONTROLLER: lib.IntelMQCLIContollerTemplate = None

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()

        cls.db_connection = cls.connect_database(POSTGRES_CONFIG)
        cls.cur = cls.db_connection.cursor(
            cursor_factory=psycopg2.extras.RealDictCursor
        )

    def setUp(self) -> None:
        super().setUp()

        self.config = {
            "database": copy.deepcopy(POSTGRES_CONFIG),
            "log_path": None,  # Do not log tests to files
            "log_level": "DEBUG",
            "rt": {
                "uri": "",
                "user": None,
                "password": None,
                "incident_report_requestor": "test-intelmq",
                "zip_threshold": 10000,
            },
            "filter": {"cc": "AT", "fqdn": ".at|.wien"},
            "tables": {
                "events": self.TEST_EVENTS_TABLE,
                "v_events_filtered": self.TEST_EVENTS_TABLE,
                "boilerplates": self.TEST_TEXT_TABLE,
            },
        }

        self.log_stream = io.StringIO(newline="\n")
        self.logger = utils.log(
            self.id(),
            log_path=False,
            stream=self.log_stream,
            log_format_stream=utils.LOG_FORMAT,
            log_level=self.config["log_level"],
        )
        self.logger_handlers_backup = self.logger.handlers

        self._clear_db_events = []
        self.addCleanup(
            lambda: self.db_delete(self.TEST_EVENTS_TABLE, self._clear_db_events)
        )

        self._prepare_rt_mock()
        rt_patch = mock.patch(
            "intelmq_extensions.cli.lib.rt.Rt", return_value=self.rt_mock
        )
        rt_patch.start()
        self.addCleanup(rt_patch.stop)

    def _prepare_rt_mock(self):
        self.rt_mock = mock.Mock(spec_set=Rt(""))
        self._rt_ticket_id = 0
        self._rt_comment_id = 0

        def _assign_id(id_name: str):
            current = getattr(self, id_name)
            setattr(self, id_name, current + 1)
            return current + 1

        self.rt_mock.create_ticket = mock.Mock(
            spec=Rt.create_ticket,
            side_effect=lambda *_, **__: _assign_id("_rt_ticket_id"),
        )
        self.rt_mock.comment = mock.Mock(
            spec=Rt.comment, side_effect=lambda *_, **__: _assign_id("_rt_comment_id")
        )

    def assertRTTicketCount(self, count: int):
        self.assertEqual(self._rt_ticket_id, count)

    def get_rt_attachment(self, investigation_id):
        for call in self.rt_mock.reply.call_args_list:
            if call.args[0] == investigation_id:
                files = call.kwargs.get("files", [])
                return files[0] if files else None
        return None

    def get_rt_text(self, investigation_id):
        for call in self.rt_mock.reply.call_args_list:
            if call.args[0] == investigation_id:
                return call.kwargs.get("text", "")
        return None

    def _print_on_failure(self):
        if sys.version_info >= (3, 11):
            # Python 3.11 removed fields used here
            return
        # Based on: https://gist.github.com/hynekcer/1b0a260ef72dae05fe9611904d7b9675
        if hasattr(self._outcome, "errors"):
            result = self.defaultTestResult()
            self._feedErrorsToResult(result, self._outcome.errors)
        else:
            result = self._outcome.result

        passed = all(test != self for test, _ in result.errors + result.failures)
        if not passed:
            for captured in ["stderr", "stdout"]:
                if data := getattr(self, captured, None):
                    print(f"{'+' * 10} CLI {captured} {'+' * 10}")
                    print(*data if isinstance(data, list) else (data.getvalue(),))

    def tearDown(self) -> None:
        self._print_on_failure()

        super().tearDown()

    def run_cli(self, args: list, expect_code=0):
        with contextlib.redirect_stderr(io.StringIO()) as f_stderr:
            with contextlib.redirect_stdout(io.StringIO()) as f_stdout:
                with unittest.mock.patch(
                    "intelmq.lib.utils.log", self.get_mocked_logger(self.logger)
                ):
                    code = self.CLI_CONTROLLER(overridden_config=self.config).run(args)

        f_stdout.seek(0)
        self.stdout = f_stdout.readlines()
        f_stderr.seek(0)
        self.stderr = f_stderr.readlines()
        self.assertEqual(expect_code, int(code))



from intelmq import HARMONIZATION_CONF_FILE
from intelmq.lib.utils import load_configuration
from mergedeep import merge


def merge_harmonization(
    additional_definitions: list[dict], harmonization_path: str = None
):
    harmonization_path = harmonization_path or HARMONIZATION_CONF_FILE
    harmonization = load_configuration(harmonization_path)

    return merge(*additional_definitions, harmonization)



"""Tests for DISP Collector

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import unittest
from unittest import mock

import intelmq.lib.message as message
import requests
from freezegun import freeze_time

from intelmq_extensions.bots.collectors.disp.collector import DISPCollectorBot

from ....base import BotTestCase
from .base import DISP_APIMockMixIn


@freeze_time("2023-09-19 12:18")
class TestDISPCollectorBot(BotTestCase, DISP_APIMockMixIn, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = DISPCollectorBot
        cls.sysconfig = {
            "api_url": "https://disp.example.at/v1",
            "auth_token": "XXX",
            "oauth_clientid": "client1",
            "oauth_clientsecret": "secret1",
            "ouath_url": "https://oauthip.example.at/v1",
            "mark_as_read": False,
            "wait_for_evidences": True,
            "check_last": "7 days",
            "code": "feed-code",
        }

    def catch_http_session(self):
        session = requests.Session()
        session_mock = mock.patch(
            "intelmq_extensions.bots.collectors.disp.collector.create_request_session",
            return_value=session,
        )
        session_mock.start()
        self.addCleanup(session_mock.stop)
        return session

    def setUp(self) -> None:
        super().setUp()
        session = self.catch_http_session()
        self.setup_api_mock(session=session)
        self._mock_incidents()

    def test_no_incidents(self):
        self.run_bot()

        self.assertOutputQueueLen(0)
        self.assertEqual(1, self.requests.call_count)

    def test_ignore_incidents_without_evidences(self):
        self.add_incident(evidence_id=None)

        self.run_bot()

        self.assertOutputQueueLen(0)
        self.assertEqual(1, self.requests.call_count)

    def test_generate_event_without_waiting_for_evidence(self):
        self.add_incident(evidence_id=None)

        self.run_bot(parameters={"wait_for_evidences": False})

        self.assertOutputQueueLen(1)

    def _create_report_dict(self, incident_data: str, evidence_data: str, **kwargs):
        report = message.Report(kwargs, harmonization=self.harmonization)
        data = {"incident": incident_data, "evidences": evidence_data}
        report.add("feed.name", "Test Bot")
        report.add("feed.accuracy", 100.0)
        report.add("feed.code", "feed-code")
        report.add("raw", json.dumps(data))
        return report.to_dict(with_type=True)

    def test_generate_event(self):
        incident_id, incident_data = self.add_incident(evidence_id="file-1")
        self.mock_evidence(incident_id, "file-1", {"my": "evidence"})

        self.run_bot()

        self.assertOutputQueueLen(1)
        self.assertMessageEqual(
            0, self._create_report_dict(incident_data, {"my": "evidence"})
        )

    def test_mark_as_read(self):
        incident_id, _ = self.add_incident(evidence_id="file-1")
        self.mock_evidence(incident_id, "file-1", {"my": "evidence"})
        self.mock_request(f"incident/read?id={incident_id}&read=true", method="post")

        self.run_bot(parameters={"mark_as_read": True})

        self.assertOutputQueueLen(1)
        self.assertIn("incident/read", self.requests.last_request.path)
        self.assertIn(f"id={incident_id}&read=true", self.requests.last_request.query)

    def test_mask_password(self):
        incident_id, incident_data = self.add_incident(evidence_id="file-1")
        self.mock_evidence(
            incident_id,
            "file-1",
            {
                "credentials": [
                    {"password": "123"},
                    {"password": "ab"},
                    {"password": ""},
                    {"password": "mysuperpassword"},
                ]
            },
        )

        self.run_bot()

        self.assertOutputQueueLen(1)
        self.assertMessageEqual(
            0,
            self._create_report_dict(
                incident_data,
                {
                    "credentials": [
                        {"password": "***"},
                        {"password": "**"},
                        {"password": ""},
                        {"password": "mys************"},
                    ]
                },
            ),
        )



"""Event Splitter can produce multiple events from a one, based
on event's tag.

Currently implemented with intention of splitting CVE events.

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import re

from intelmq.lib.bot import ExpertBot


class EventSplitterExpertBot(ExpertBot):
    look_in: str = "extra.tag"
    copy_to: list[str] = ["classification.identifier", "extra.vulnerabilities"]
    regex: str = ""  # has to match one group meaning matched duplication value

    def init(self):
        self.matcher = re.compile(self.regex)

    def process(self):
        event = self.receive_message()

        lookup_data = event.get(self.look_in, "")
        matches = self.matcher.findall(lookup_data)
        if not matches:
            self.send_message(event)
        else:
            for matched in matches:
                self.logger.debug("Found: %s.", matched)
                sub_event = self.new_event(event)
                for key in self.copy_to:
                    sub_event.add(key, matched, overwrite=True)
                self.send_message(sub_event)

        self.acknowledge_message()


BOT = EventSplitterExpertBot



"""Common parts of BlackKite integration

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

from enum import Enum


class Category(str, Enum):
    ApplicationSecurity = "APPSEC"
    PatchManagement = "PATCH"
    DNSHealth = "DNS"
    EmailSecurity = "SMTP"
    FraudulentDomains = "FRADOM"
    CredentialManagement = "LEAK"
    IPReputation = "REP"
    NetworkSecurity = "NETSEC"
    CDNSecurity = "CDN"
    DDoSResiliency = "DDOS"
    AttackSurface = "ATTSRF"
    SSLTLSstrength = "SSL"
    FraudulentApps = "FRAAPP"
    InformationDisclosure = "INFDIS"
    HacktivistShares = "HACK"
    SocialNetwork = "SOCIAL"
    BrandMonitoring = "BRM"
    WebRanking = "WBR"
    WebsiteSecurity = "WSS"



# -*- coding: utf-8 -*-
"""
Testing Copy Extra expert bot.
"""

import unittest

from intelmq_extensions.bots.experts.copy_extra.expert import CopyExtraExpertBot

from ....base import BotTestCase

INPUT = {
    "__type": "Event",
    "time.observation": "2015-01-01T00:00:00+00:00",
    "extra.deviceid": "foo",
    "extra.devicerev": "bar",
    "extra.firmwarerev": 1,
}
OUTPUT = INPUT.copy()
OUTPUT["shareable_extra_info"] = '{"deviceid": "foo", "firmwarerev": 1}'


class TestCopyExtraExpertBot(BotTestCase, unittest.TestCase):
    """
    A TestCase for CopyExtraExpertBot.
    """

    @classmethod
    def set_bot(cls):
        cls.bot_reference = CopyExtraExpertBot
        cls.sysconfig = {"keys": ["deviceid", "firmwarerev"]}
        cls.default_input_message = {"__type": "Event"}

    def test_events(self):
        """Test if correct Events have been produced."""
        self.input_message = INPUT
        self.run_bot()
        self.assertMessageEqual(0, OUTPUT)


if __name__ == "__main__":
    unittest.main()



"""
XMPP Output Bot
Connects to a XMPP Server and sends data to a user.

TLS is used by default.

Tested with Python >= 3.4
Tested with slixmpp >= 1.0.0-beta5

Copyright (C) 2016 by Bundesamt für Sicherheit in der Informationstechnik
Software engineering by Intevation GmbH

Parameters:
ca_certs: string to a CA-bundle file or false/empty string for no checks
hierarchical_output: boolean (false by default)
xmpp_user: string
xmpp_server: string
xmpp_password: boolean
xmpp_to_user: string
xmpp_to_server: string
xmpp_room: string
xmpp_room_nick: string
xmpp_room_password: string
use_muc: boolean
"""

from intelmq.lib.bot import Bot
from intelmq.lib.exceptions import MissingDependencyError

try:
    import slixmpp

    class XMPPClient(slixmpp.ClientXMPP):
        def __init__(self, jid, password, room, room_nick, room_password, logger):
            slixmpp.ClientXMPP.__init__(self, jid, password)

            self.xmpp_room = room
            self.xmpp_room_nick = room_nick
            self.xmpp_room_password = room_password

            self.logger = logger
            self.add_event_handler("session_start", self.session_start)

            self.logger.info("Initialized XMPP Client.")

        def session_start(self, event):
            self.send_presence()
            self.logger.debug("Session started.")

            try:
                self.get_roster()
            except slixmpp.exceptions.IqError as err:
                self.logger.error("There was an error getting the roster.")
                self.logger.error(err.iq["error"]["condition"])
                self.disconnect()
            except slixmpp.exceptions.IqTimeout:
                self.logger.error("Server is taking too long to respond.")
                self.disconnect()

            if (
                self.xmpp_room
            ):  # and self.plugin.get('xep_0045') # this check should also exist!
                self.logger.debug("Joining room: %s.", self.xmpp_room)
                pwd = self.xmpp_room_password if self.xmpp_room_password else ""
                self.plugin["xep_0045"].joinMUC(
                    self.xmpp_room, self.xmpp_room_nick, password=pwd, wait=True
                )

except ImportError:
    slixmpp = None


class XMPPOutputBot(Bot):
    xmpp = None

    def init(self):
        self.logger.warning(
            "The output bot 'intelmq.bots.outputs.xmpp.output' "
            "is deprecated. It will be removed in version 3.0."
            "Please see https://github.com/certtools/intelmq/blob/"
            "develop/NEWS.md#xmpp-bots for more details."
        )
        if slixmpp is None:
            raise MissingDependencyError("slixmpp")

        # Retrieve Parameters from configuration
        xmpp_user = getattr(self.parameters, "xmpp_user", None)
        xmpp_server = getattr(self.parameters, "xmpp_server", None)
        xmpp_password = getattr(self.parameters, "xmpp_password", None)

        if None in (xmpp_user, xmpp_server, xmpp_password):
            raise ValueError("No User / Password provided.")
        else:
            xmpp_login = xmpp_user + "@" + xmpp_server

        self.muc = getattr(self.parameters, "use_muc", None)
        xmpp_to_user = getattr(self.parameters, "xmpp_to_user", None)
        xmpp_to_server = getattr(self.parameters, "xmpp_to_server", None)
        xmpp_room = getattr(self.parameters, "xmpp_room", None) if self.muc else None
        xmpp_room_nick = (
            getattr(self.parameters, "xmpp_room_nick", None) if self.muc else None
        )
        xmpp_room_password = (
            getattr(self.parameters, "xmpp_room_password", None) if self.muc else None
        )

        ca_certs = getattr(self.parameters, "ca_certs", None)

        # Be sure the receiver was set up
        if not self.muc and None in (xmpp_to_user, xmpp_to_server):
            raise ValueError("No receiver for direct messages provided.")
        else:
            self.xmpp_receiver = xmpp_to_user + "@" + xmpp_to_server

        if self.muc and not xmpp_room:
            raise ValueError("No room provided.")
        else:
            self.xmpp_receiver = xmpp_room

        if self.muc:
            if not xmpp_room_nick:
                # create the room_nick from user and server
                xmpp_room_nick = xmpp_login

        self.xmpp = XMPPClient(
            xmpp_login,
            xmpp_password,
            xmpp_room,
            xmpp_room_nick,
            xmpp_room_password,
            self.logger,
        )

        if ca_certs:
            # Set CA-Certificates
            self.xmpp.ca_certs = ca_certs

        if self.xmpp.connect(reattempt=False):
            self.xmpp.process()
            # Add Handlers and register Plugins
            self.xmpp.register_plugin("xep_0030")  # Service Discovery
            self.xmpp.register_plugin("xep_0045")  # Multi-User Chat
        else:
            raise ValueError("Could not connect to XMPP-Server.")

    def process(self):
        event = self.receive_message()

        jevent = event.to_json(
            hierarchical=self.parameters.hierarchical_output, with_type=True
        )

        try:
            # TODO: proper error handling.
            # Right now it cannot be detected if the message was sent successfully.
            if self.muc:
                self.logger.debug("Trying to send to room %s.", self.xmpp_receiver)
                self.xmpp.send_message(
                    mto=self.xmpp_receiver, mbody=jevent, mtype="groupchat"
                )
            else:
                self.logger.debug("Trying to send to %s.", self.xmpp_receiver)
                self.xmpp.send_message(mto=self.xmpp_receiver, mbody=jevent)
        except slixmpp.exceptions.XMPPError as err:
            self.logger.error("There was an error when sending the event.")
            self.logger.error(err.iq["error"]["condition"])

        self.acknowledge_message()

    def shutdown(self):
        if self.xmpp:
            if self.xmpp.disconnect():
                self.logger.info("Disconnected from XMPP Server.")
            else:
                self.logger.error("Could not disconnect from XMPP Server.")
        else:
            self.logger.info("There was no XMPPClient I could stop.")


BOT = XMPPOutputBot



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import os
import os.path
import unittest
from copy import deepcopy
from unittest import mock

import intelmq.lib.test as test
import pkg_resources

from intelmq_extensions.bots.experts.squelcher.expert import SquelcherExpertBot

from ....base import POSTGRES_CONFIG, BotTestCase

INPUT1 = {
    "__type": "Event",
    "classification.identifier": "zeus",
    "classification.type": "infected-system",
    "notify": False,
    "source.asn": 1,
    "source.ip": "192.0.2.1",
    "feed.name": "Example Feed",
}

INPUT2 = INPUT1.copy()
INPUT2["classification.identifier"] = "https"
INPUT2["classification.type"] = "vulnerable-system"
OUTPUT2 = INPUT2.copy()
OUTPUT2["notify"] = True

INPUT3 = INPUT1.copy()
INPUT3["classification.identifier"] = "https"
INPUT3["classification.type"] = "vulnerable-system"
INPUT3["source.ip"] = "192.0.2.4"

INPUT4 = INPUT3.copy()
INPUT4["classification.identifier"] = "openresolver"
INPUT4["notify"] = True

INPUT5 = INPUT4.copy()
INPUT5["source.ip"] = "198.51.100.5"
OUTPUT5 = INPUT5.copy()
OUTPUT5["notify"] = False

INPUT6 = INPUT4.copy()
INPUT6["source.ip"] = "198.51.100.45"
OUTPUT6 = INPUT6.copy()
OUTPUT6["notify"] = False

INPUT7 = INPUT1.copy()
INPUT7["notify"] = True
INPUT7["source.fqdn"] = "example.com"
del INPUT7["source.ip"]
OUTPUT7 = INPUT7.copy()

INPUT8 = INPUT1.copy()
del INPUT8["notify"]
del INPUT8["source.asn"]
OUTPUT8 = INPUT8.copy()
OUTPUT8["notify"] = False

INPUT_INFINITE = {
    "__type": "Event",
    "classification.identifier": "zeus",
    "classification.type": "infected-system",
    "source.asn": 12312,
    "source.ip": "192.0.2.1",
}
OUTPUT_INFINITE = INPUT_INFINITE.copy()
OUTPUT_INFINITE["notify"] = False

INPUT_RANGE = {
    "__type": "Event",
    "classification.identifier": "zeus",
    "classification.type": "infected-system",
    "source.asn": 789,
    "source.ip": "10.0.0.10",
}

INPUT9 = INPUT1.copy()
INPUT9["extra.additionalmetadata"] = ["foobar"]

INPUT10 = INPUT1.copy()
INPUT10["notify"] = True

INPUT11 = INPUT1.copy()
INPUT11["extra.malware.variants"] = ["foo", "bar"]


# TODO: move test case to the use the test DB helper


@test.skip_database()
@test.skip_exotic()
class TestSquelcherExpertBot(BotTestCase, unittest.TestCase):
    """
    A TestCase for SquelcherExpertBot.
    """

    @classmethod
    def set_bot(cls):
        cls.bot_reference = SquelcherExpertBot
        cls.default_input_message = INPUT1
        if not os.environ.get("INTELMQ_TEST_DATABASES"):
            return
        cls.sysconfig = {
            "configuration_path": pkg_resources.resource_filename(
                "intelmq_extensions", "etc/squelcher.conf"
            ),
            "overwrite": True,
            "sending_time_interval": "2 years",
            "table": cls.TEST_EVENTS_TABLE,
            "logging_level": "DEBUG",
        }
        cls.sysconfig.update(POSTGRES_CONFIG)
        cls.con = cls.connect_database(POSTGRES_CONFIG)
        cls.con.autocommit = True
        cls.cur = cls.con.cursor()
        cls.truncate(cls)

    def truncate(self):
        self.cur.execute("TRUNCATE TABLE {}".format(self.sysconfig["table"]))

    def insert(
        self,
        classification_identifier,
        classification_type,
        notify,
        source_asn,
        source_ip,
        time_source,
        sent_at=None,
        report_id=None,
        feed_name="Example Feed",
        extra=None,
    ):
        if sent_at is not None:
            append = "LOCALTIMESTAMP + INTERVAL %s second"
        else:
            append = "%s"
        query = """
INSERT INTO {table}(
    "classification.identifier", "classification.type", notify, "source.asn",
    "source.ip", "time.source", "rtir_report_id", "sent_at", "feed.name", "extra"
) VALUES (%s, %s, %s, %s, %s,
    LOCALTIMESTAMP + INTERVAL %s second, %s,
    {append}, %s, %s)
""".format(table=self.sysconfig["table"], append=append)
        self.cur.execute(
            query,
            (
                classification_identifier,
                classification_type,
                notify,
                source_asn,
                source_ip,
                time_source,
                report_id,
                sent_at,
                feed_name,
                json.dumps(extra or dict()),
            ),
        )

    def test_ttl_1(self):
        "event exists in db -> squelch"
        self.insert("zeus", "infected-system", True, 1, "192.0.2.1", "0")
        self.input_message = INPUT1
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT1)

    def test_ttl_2(self):
        "event in db is too old -> notify"
        self.insert("https", "vulnerable-system", True, 1, "192.0.2.1", "- 01:45")
        self.input_message = INPUT2
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 3600 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT2)

    def test_ttl_2h_squelch(self):
        "event is in db -> squelch"
        self.insert("https", "vulnerable-system", True, 1, "192.0.2.4", "- 01:45")
        self.input_message = INPUT3
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 7200 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT3)

    def test_network_match(self):
        """event is in db without notify -> notify
        find ttl based on network test"""
        self.insert(
            "openresolver", "vulnerable-system", False, 1, "198.51.100.5", "- 20:00"
        )
        self.input_message = INPUT5
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 115200 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT5)

    def test_network_match3(self):
        """event is in db -> squelch
        find ttl based on network test"""
        self.insert(
            "openresolver",
            "vulnerable-system",
            True,
            1,
            "198.51.100.5",
            "- 25:00",
            "- 25:00",
        )
        self.input_message = INPUT5
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 115200 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT5)

    def test_address_match1(self):
        "event in db is too old -> notify"
        self.insert(
            "openresolver",
            "vulnerable-system",
            True,
            1,
            "198.51.100.45",
            "- 25:00",
            "- 25:00",
        )
        self.input_message = INPUT6
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 86400 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT6)

    def test_address_match2(self):
        "event is in db -> squelch"
        self.insert(
            "openresolver",
            "vulnerable-system",
            True,
            1,
            "198.51.100.45",
            "- 20:00",
            "- 20:00",
        )
        self.input_message = INPUT6
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 86400 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT6)

    def test_ttl_other_ident(self):
        "other event in db -> notify"
        self.insert(
            "https", "vulnerable-system", True, 1, "198.51.100.5", "- 01:45", "- 01:45"
        )
        self.input_message = INPUT4
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 7200 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT4)

    def test_use_ttl_from_event(self):
        "Honour TTL from the event"
        input_msg = INPUT4.copy()
        input_msg["source.ip"] = "82.82.82.82"

        # Default TTL
        self.input_message = input_msg
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")
        self.assertMessageEqual(0, input_msg)

        # Pre-defined TTL
        input_msg["extra.ttl"] = 10
        self.input_message = input_msg
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 10 for", levelname="DEBUG")
        self.assertMessageEqual(0, input_msg)

    def test_domain(self):
        "only domain -> notify true"
        self.input_message = INPUT7
        self.run_bot()
        self.truncate()
        self.assertNotRegexpMatchesLog("Found TTL")
        self.assertMessageEqual(0, OUTPUT7)

    def test_missing_asn(self):
        "no asn -> notify false"
        self.input_message = INPUT8
        self.run_bot()
        self.truncate()
        self.assertNotRegexpMatchesLog("Found TTL")
        self.assertMessageEqual(0, OUTPUT8)

    def test_domain_when_non_ip_filter(self):
        "only domain, but filtering without IP allowed -> process"
        self.input_message = INPUT7
        self.run_bot(parameters={"filter_ip_only": False, "source_fields": "feed.name"})
        self.truncate()
        self.assertRegexpMatchesLog("Found TTL 604800")  # default ttl
        self.assertMessageEqual(0, OUTPUT7)

    def test_missing_asn_when_non_ip_filter(self):
        "no asn, but filtering without IP allowed -> process"
        self.input_message = INPUT8
        self.run_bot(parameters={"filter_ip_only": False, "source_fields": "feed.name"})
        self.truncate()
        self.assertRegexpMatchesLog("Found TTL 604800")  # default ttl
        output = OUTPUT8.copy()
        output["notify"] = True
        self.assertMessageEqual(0, output)

    def test_infinite(self):
        "never notify with ttl -1"
        self.input_message = INPUT_INFINITE
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL -1 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT_INFINITE)

    def test_iprange(self):
        "test if mechanism checking IP ranges"
        self.input_message = INPUT_RANGE
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 72643 for", levelname="DEBUG")

    def test_unsent_notify(self):
        """event exists, but is older than 1 day and has not been sent -> notify"""
        self.insert(
            "openresolver",
            "vulnerable-system",
            True,
            1,
            "198.51.100.5",
            str(-25 * 3600),
        )
        self.sysconfig["sending_time_interval"] = "1 day"
        self.input_message = INPUT5
        self.run_bot()
        self.sysconfig["sending_time_interval"] = "2 days"
        self.truncate()
        self.assertLogMatches("Found TTL 115200 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT5)

    def test_unsent_squelch(self):
        """event exists, is younger than 2 days and has not been sent -> squelch"""
        self.insert(
            "openresolver", "vulnerable-system", True, 1, "198.51.100.5", "- 86400"
        )
        self.input_message = INPUT5
        self.run_bot()
        self.truncate()
        self.assertLogMatches("Found TTL 115200 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT5)

    def test_extra_list(self):
        """lists in extra data is handled."""
        self.insert("zeus", "infected-system", True, 1, "192.0.2.1", "0")
        self.input_message = INPUT9
        self.run_bot()
        self.truncate()
        self.assertMessageEqual(0, INPUT9)

    def test_overwrite_false(self):
        """check if notify is not overwritten if not allowed."""
        self.input_message = INPUT10
        self.sysconfig["overwrite"] = False
        self.run_bot()
        self.sysconfig["overwrite"] = True
        self.assertLogMatches(
            "Notify field present and not allowed to overwrite, skipping.",
            levelname="DEBUG",
        )
        self.assertMessageEqual(0, INPUT10)

    def test_hashable(self):
        self.input_message = INPUT11
        self.run_bot(
            parameters={
                "configuration_path": os.path.join(
                    os.path.dirname(__file__), "unhashable.config"
                )
            }
        )
        self.assertLogMatches("Found TTL 123 for", levelname="DEBUG")

    def test_open_report_ttl_squelch(self):
        "event with report exists in db -> squelch"
        self.insert("zeus", "infected-system", True, 1, "192.0.2.1", "0", report_id="1")
        self.input_message = INPUT1
        self.run_bot(parameters={"query": "open_report"})
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT1)

    def test_open_report_ttl_too_old(self):
        "event in db is too old -> notify"
        self.insert(
            "https", "vulnerable-system", True, 1, "192.0.2.1", "- 01:45", report_id="1"
        )
        self.input_message = INPUT2
        self.run_bot(parameters={"query": "open_report"})
        self.truncate()
        self.assertLogMatches("Found TTL 3600 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT2)

    def test_custom_sources_ttl_too_old(self):
        "event in db is too old, matched using custom field -> notify"
        # different IP
        self.insert("https", "vulnerable-system", True, 1, "0.0.0.0", "- 01:45")
        self.input_message = INPUT2
        self.run_bot(parameters={"source_fields": "feed.name,source.asn"})
        self.truncate()
        self.assertLogMatches("Found TTL 3600 for", levelname="DEBUG")
        self.assertMessageEqual(0, OUTPUT2)

    def test_custom_sources_ttl_squelch(self):
        "event with report exists in db -> squelch"
        self.insert("zeus", "infected-system", True, 1, "0.0.0.0", "0")
        self.input_message = INPUT1
        self.run_bot(parameters={"source_fields": "feed.name,source.asn"})
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")
        self.assertMessageEqual(0, INPUT1)

    def test_custom_sources_json_squelch(self):
        "event with report exists in db -> squelch"
        self.insert(
            "zeus",
            "infected-system",
            True,
            1,
            "0.0.0.0",
            "0",
            extra={"ident": "something"},
        )
        message = INPUT1.copy()
        message["extra.ident"] = "something"
        self.input_message = deepcopy(message)
        self.run_bot(parameters={"source_fields": "extra.ident"})
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")
        self.assertMessageEqual(0, message)

    def test_custom_sources_all_need_match(self):
        "all custom source fields need to match to squelch"
        self.insert("zeus", "infected-system", True, 2, "0.0.0.0", "0")
        self.insert(
            "zeus", "infected-system", True, 1, "0.0.0.0", "0", feed_name="Another"
        )
        self.input_message = INPUT1
        self.run_bot(parameters={"source_fields": "feed.name,source.asn"})
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")
        output = INPUT1.copy()
        output["notify"] = True
        self.assertMessageEqual(0, output)

    def test_when_source_field_is_null(self):
        "Squelcher should compare against null values"
        self.insert("zeus", "infected-system", True, 2, "0.0.0.0", "0", report_id=None)
        self.input_message = INPUT1

        self.run_bot(parameters={"source_fields": "feed.name,rtir_report_id"})
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")

        # Notify: False - the same feed name and both events has lack of report id
        self.assertMessageEqual(0, INPUT1)

    def test_when_json_source_field_is_null(self):
        self.insert("zeus", "infected-system", True, 2, "0.0.0.0", "0")
        self.input_message = INPUT1

        self.run_bot(parameters={"source_fields": "feed.name,extra.something"})
        self.truncate()
        self.assertLogMatches("Found TTL 604800 for", levelname="DEBUG")

        # Notify: False - the same feed name and both events has lack of extra.something
        self.assertMessageEqual(0, INPUT1)

    def test_static_bot_check_method(self, *args, **kwargs):
        with mock.patch(
            "intelmq.lib.utils.load_configuration", new=test.mocked_config()
        ):
            super().test_static_bot_check_method()

    @classmethod
    def tearDownClass(cls):
        if not os.environ.get("INTELMQ_TEST_DATABASES"):
            return
        cls.truncate(cls)
        cls.cur.close()
        cls.con.close()


class TestSquelcherExpertBotHelper(unittest.TestCase):
    def test_convert_config_list(self):
        self.config = [
            [
                {"extra.malware.variants": ["foo", "bar"]},
                {"ttl": 10},
            ]
        ]
        SquelcherExpertBot.convert_config(self)
        self.assertEqual(
            self.config,
            [
                [
                    {"extra.malware.variants": ("foo", "bar")},
                    {"ttl": 10},
                ]
            ],
        )

    def test_convert_config_dict(self):
        self.config = [
            [
                {"extra.malware.variants": {"foo": "bar"}},
                {"ttl": 10},
            ]
        ]
        SquelcherExpertBot.convert_config(self)
        self.assertEqual(
            self.config,
            [
                [
                    {"extra.malware.variants": (("foo", "bar"),)},
                    {"ttl": 10},
                ]
            ],
        )


if __name__ == "__main__":
    unittest.main()



"""Tests for BlackKiteParserBot

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import unittest
from datetime import datetime

import intelmq.lib.message as message
from dateutil.tz import UTC

from intelmq_extensions.bots.parsers.blackkite.parser import BlackKiteParserBot

from ....base import BotTestCase
from .data import (
    APPLICATION_SECURITY,
    CREDENTIAL_MANAGEMENT,
    DEFAULT_COMPANY,
    PATCH_MANAGEMENT,
)


class TestBlackKiteParserBot(BotTestCase, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = BlackKiteParserBot
        cls.sysconfig = {}

    def _create_report_dict(self, finding_data: dict, company_data: dict = None):
        company_data = company_data or DEFAULT_COMPANY
        raw = {"company": company_data, "finding": finding_data}

        data = {
            "feed.accuracy": 100.0,
            "time.observation": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
        }

        report = message.Report(data, harmonization=self.harmonization)
        report.add("raw", json.dumps(raw))
        return report.to_dict(with_type=True)

    def _create_event_dict(
        self,
        ip: str = "127.0.0.1",
        fqdn: str = None,
        event_id: int = 1,
        monitored_asset: str = "cert.at",
        identifier: str = "bk-patch",
        taxonomy: str = "vulnerable",
        type_: str = "vulnerable-system",
        **kwargs,
    ):
        data = {
            "classification.identifier": identifier,
            "classification.taxonomy": taxonomy,
            "classification.type": type_,
            "source.fqdn": fqdn,
            "source.ip": ip,
            "extra.feed_event_id": event_id,
            "extra.blackkite_company_id": 1,
            "feed.accuracy": 100.0,
            "extra.monitored_asset": monitored_asset,  # Domain from Company
            "time.observation": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
            **kwargs,
        }
        event = message.Event(data, harmonization=self.harmonization)
        if "time.source" not in event:
            event.add("time.source", "2023-08-28T13:37:22.717")
        event.add("raw", json.dumps({"company": {}, "finding": {}}))
        return event.to_dict(with_type=True)

    def test_parse_patch_management_finding(self):
        self.input_message = self._create_report_dict(PATCH_MANAGEMENT)

        self.run_bot()

        expected = {
            "identifier": "cve-2023-38408",
            "event_description.text": PATCH_MANAGEMENT["Detail"],
            "event_description.url": "https://nvd.nist.gov/vuln/detail/CVE-2023-38408",
            "extra.vendor": "openbsd",
            "extra.product": "openssh",
            "extra.product_name": "openssh_/5.9",
            "extra.vulnerabilities": "cve-2023-38408",
            # Cvss score? Severity?
            "feed.documentation": "https://cyber.riskscore.cards/kb/PATCH-001",
            "feed.code": "blackkite-patch",
            "feed.name": "BlackKite PATCH",
        }
        self.assertMessageEqual(
            0, self._create_event_dict(**expected), compare_raw=False
        )

    def test_handling_domain_in_ip_field(self):
        "This is a workaround for bug in BlackKite, already reported. See Taiga#808"
        msg = PATCH_MANAGEMENT.copy()
        msg["IpAddress"] = "override.example.at"
        self.input_message = self._create_report_dict(msg)

        self.run_bot()

        expected = {
            "source.ip": None,
            # "source.fqdn": "override.example.at", FIXME: Wait for explanation from BlackKite
            "identifier": "cve-2023-38408",
            "event_description.text": PATCH_MANAGEMENT["Detail"],
            "event_description.url": "https://nvd.nist.gov/vuln/detail/CVE-2023-38408",
            "extra.vendor": "openbsd",
            "extra.product": "openssh",
            "extra.product_name": "openssh_/5.9",
            "extra.vulnerabilities": "cve-2023-38408",
            "feed.documentation": "https://cyber.riskscore.cards/kb/PATCH-001",
            "feed.code": "blackkite-patch",
            "feed.name": "BlackKite PATCH",
        }
        self.assertMessageEqual(
            0, self._create_event_dict(**expected), compare_raw=False
        )

    def test_parse_patch_management_finding_without_cve(self):
        # e.g. EoL software
        msg = PATCH_MANAGEMENT.copy()
        msg["ControlId"] = "PATCH-010"
        msg["CveId"] = None
        self.input_message = self._create_report_dict(msg)

        self.run_bot()

        expected = {
            "identifier": "end-of-live",
            "event_description.text": msg["Detail"],
            "event_description.url": "https://nvd.nist.gov/vuln/detail/CVE-2023-38408",
            "extra.vendor": "openbsd",
            "extra.product": "openssh",
            "extra.product_name": "openssh_/5.9",
            # Cvss score? Severity?
            "feed.documentation": "https://cyber.riskscore.cards/kb/PATCH-010",
            "feed.code": "blackkite-patch",
            "feed.name": "BlackKite PATCH",
        }
        self.assertMessageEqual(
            0, self._create_event_dict(**expected), compare_raw=False
        )

    def test_parse_application_security_finding(self):
        self.input_message = self._create_report_dict(APPLICATION_SECURITY)

        self.run_bot()

        expected = {
            "classification.taxonomy": "vulnerable",
            "classification.type": "potentially-unwanted-accessible",
            "identifier": "bk-appsec",
            "event_description.text": (
                "Cleartext Transmission of Sensitive Information. "
                "Not encrypted communication on xxx."
            ),
            "feed.documentation": "https://cyber.riskscore.cards/kb/APPSEC-014",
            "extra.feed_event_id": 658443,
            "source.ip": None,
            "feed.code": "blackkite-appsec",
            "feed.name": "BlackKite APPSEC",
        }
        self.assertMessageEqual(
            0, self._create_event_dict(**expected), compare_raw=False
        )

    def test_parse_credential_management_finding(self):
        self.input_message = self._create_report_dict(CREDENTIAL_MANAGEMENT)

        self.run_bot()

        expected = {
            "classification.taxonomy": "information-content-security",
            "classification.type": "data-leak",
            "classification.identifier": "leaked-credentials",
            "source.fqdn": "example.cert.at",
            "extra.monitored_asset": "cert.at",
            "feed.documentation": "https://cyber.riskscore.cards/kb/LEAK-003",
            "source.account": "user@example.cert.at",
            "extra.account": "user@example.cert.at",  # intentionally twice
            "extra.password": "PLAIN",  # BK doesn't provide us more info
            "extra.compromise_time_full": "2022-09-22T00:00:00",
            "extra.compromise_time": "2022-09",
            "extra.leak_source": "cert_com_leak",
            "source.ip": None,
            "extra.feed_event_id": 44823698,
            # This feed doesn't have a proper source time, use observation instead
            "time.source": datetime(2023, 1, 1, tzinfo=UTC).isoformat(),
            "feed.code": "blackkite-leak",
            "feed.name": "BlackKite LEAK",
        }
        self.assertMessageEqual(
            0, self._create_event_dict(**expected), compare_raw=False
        )



"""Collector for Deloitte Intelligence Service Portal

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later

Connects to the DISP portal and collects selected type of incidents
for every monitored company. Currently, we are prepared for the credential tracing
only.

Parameters:
api_url
auth_token
oauth_clientid
oauth_clientsecret
ouath_url

mark_as_read
wait_for_evidences
mask_password # hardcoded to true
"""

import json
from datetime import datetime, timedelta

from intelmq.lib.bot import CollectorBot
from intelmq.lib.utils import create_request_session, parse_relative

from ._client import DISPClient


class DISPCollectorBot(CollectorBot):
    api_url: str = ""
    auth_token: str = ""
    oauth_clientid: str = ""
    oauth_clientsecret: str = ""
    ouath_url: str = ""
    # refresh access token when it's less than 10 minutes to expire
    refresh_token_before: int = 10

    mark_as_read: bool = False
    wait_for_evidences: bool = True
    not_older_than: str = "7 days"
    # mask_password: bool = True  # hardcoded to true

    def init(self):
        self.set_request_parameters()
        self.session = create_request_session(self)
        self.client = DISPClient(
            api_url=self.api_url,
            auth_token=self.auth_token,
            oauth_clientid=self.oauth_clientid,
            oauth_clientsecret=self.oauth_clientsecret,
            oauth_url=self.ouath_url,
            session=self.session,
            refresh_before=self.refresh_token_before,
            logger=self.logger,
        )

    @staticmethod
    def _mask_passwords(data):
        """Ensure passwords will never be processed nor stored in our systems"""
        for credentials in data.get("credentials", []):
            password = credentials.get("password")
            if not password:
                continue
            if len(password) <= 3:
                mask = "*" * len(password)
            else:
                mask = f"{password[:3]}{'*' * (len(password) - 3)}"
            credentials["password"] = mask

    def process(self):
        after = datetime.utcnow() - timedelta(
            minutes=parse_relative(self.not_older_than)
        )
        for incident in self.client.incidents(after=after, only_unread=True):
            evidences = incident.get("evidences", [])
            expected_file = f'{incident["id"]}.json.txt'
            evidence_file = next(
                filter(lambda f: f.get("name") == expected_file, evidences), None
            )
            if self.wait_for_evidences and not evidence_file:
                self.logger.debug(
                    "Incident %s doesn't have an evidence file yet.", incident.get("id")
                )
                continue

            evidence_data = None
            if evidence_file:
                evidence_data = self.client.download_evidence_json(
                    incident.get("id"), evidence_file.get("idStoredFile")
                )
                self._mask_passwords(evidence_data)

            report = self.new_report()
            report.add(
                "raw", json.dumps({"incident": incident, "evidences": evidence_data})
            )
            self.send_message(report)
            if self.mark_as_read:
                self.client.mark_incident_read(incident.get("id"))


BOT = DISPCollectorBot



from intelmq.lib.test import skip_database

from intelmq_extensions.cli.intelmqcli import IntelMQCLIContoller

from ..base import CLITestCase


@skip_database()
class TestIntelMQCLI(CLITestCase):
    CLI_CONTROLLER = IntelMQCLIContoller

    def test_list_feeds(self):
        self.db_add_event({"feed.code": "my-feed"})
        self.db_add_event({"feed.code": "feed-2"})

        self.run_cli(["-l"])

        self.assertIn("my-feed\n", self.stdout)
        self.assertIn("feed-2\n", self.stdout)

    def test_add_event_description_to_text(self):
        self.add_boilerplate(
            "test", "my generic description and \n{event_descriptions}"
        )
        self.add_boilerplate("event-description-divider", "\n ### \n")
        self.db_add_event(
            {
                "feed.code": "feed1",
                "rtir_report_id": 10,
                "event_description.text": "descrpt1",
            }
        )
        self.db_add_event(
            {
                "feed.code": "feed1",
                "rtir_report_id": 10,
                "event_description.text": "description with \\n encoded new line",
            }
        )

        self.run_cli(["--batch", "--quiet"])

        self.assertRTTicketCount(2)

        text = self.get_rt_text(2)
        self.assertIn(
            "description with \n encoded new line",
            text,
        )
        self.assertIn("\n ### \n", text)
        self.assertIn("descrpt1", text)

    def test_add_event_description_to_text_default_divider(self):
        self.add_boilerplate(
            "test", "my generic description and \n{event_descriptions}"
        )
        self.db_add_event(
            {
                "feed.code": "feed1",
                "rtir_report_id": 10,
                "event_description.text": "descrpt1",
            }
        )
        self.db_add_event(
            {
                "feed.code": "feed1",
                "rtir_report_id": 10,
                "event_description.text": "description with \\n encoded new line",
            }
        )

        self.run_cli(["--batch", "--quiet"])

        self.assertRTTicketCount(2)

        text = self.get_rt_text(2)
        self.assertIn("\n --- \n", text)


@skip_database()
class TestConstituencySupport(CLITestCase):
    CLI_CONTROLLER = IntelMQCLIContoller

    def setUp(self) -> None:
        super().setUp()
        self.config.update({"constituency": {"key": "energy"}})

        self.add_boilerplate("test", "some_body")

        self.event_ids = [
            self.db_add_event(
                {"feed.code": "feed1", "constituency": "energy", "rtir_report_id": 10}
            ),
            self.db_add_event(
                {"feed.code": "feed2", "constituency": "energy", "rtir_report_id": 20}
            ),
            self.db_add_event(
                {"feed.code": "feed3", "constituency": "national", "rtir_report_id": 30}
            ),
            self.db_add_event(
                {"feed.code": "feed4", "constituency": "national", "rtir_report_id": 40}
            ),
            self.db_add_event(
                {"feed.code": "feed5", "constituency": None, "rtir_report_id": 50}
            ),
        ]

    def test_all_list_feeds(self):
        """intelmqcli ignore constituency when listing - FIXME"""
        self.run_cli(["-l"])

        self.assertEqual(5, len(self.stdout))

    def test_run_create_reports(self):
        self.run_cli(["--batch", "--quiet"])

        self.assertInLogs("All taxonomies: test")

        # 1 new incident + 1 new investigation
        # events have the same abuse contact and taxonomy, so they are processed together
        self.assertRTTicketCount(2)

        for event_id in self.event_ids[:2]:
            event = self.db_get_event(event_id)
            self.assertEqual(event["rtir_incident_id"], 1)
            self.assertEqual(event["rtir_investigation_id"], 2)
            self.assertIsNotNone(event["sent_at"])

        # Other events untouched
        for event_id in self.event_ids[2:]:
            event = self.db_get_event(event_id)
            self.assertIsNone(event["rtir_incident_id"])
            self.assertIsNone(event["rtir_investigation_id"])
            self.assertIsNone(event["sent_at"])

    def test_creating_attachment_removes_new_lines_from_description(self):
        self.db_add_event(
            {
                "feed.code": "feed2",
                "constituency": "energy",
                "rtir_report_id": 20,
                "event_description.text": (
                    "Some very long description \n with \n\n multiple \n new lines."
                ),
            }
        )

        self.run_cli(["--batch", "--quiet"])

        self.assertInLogs("All taxonomies: test")

        self.assertRTTicketCount(2)

        _, content_io, __ = self.get_rt_attachment(investigation_id=2)
        content: str = content_io.getvalue()
        self.assertIn(
            "Some very long description   with    multiple   new lines.", content
        )



"""Testing events splitter

SPDX-FileCopyrightText: 2025 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
import os
import unittest
from unittest import mock

import intelmq.lib.test as test
import requests
from requests_mock import MockerCore

from intelmq_extensions.bots.experts.vulnerability_lookup.expert import (
    VulnerabilityLookupExpertBot,
)

from ....base import BotTestCase

INPUT = {
    "__type": "Event",
    "classification.identifier": "zeus",
    "classification.type": "vulnerable-system",
    "notify": True,
}


@test.skip_redis()
class TestVulnerabilityLookupExpertBot(BotTestCase, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.use_cache = True
        cls.bot_reference = VulnerabilityLookupExpertBot

    def mock_http_session(self):
        session = requests.Session()
        session_mock = mock.patch(
            "intelmq_extensions.bots.experts.vulnerability_lookup.expert.create_request_session",
            return_value=session,
        )
        session_mock.start()
        self.addCleanup(session_mock.stop)

        self.requests = MockerCore(session=session)
        self.requests.start()
        self.addCleanup(self.requests.stop)

    def _load_data_file(self, filename: str):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.join(current_dir, "data", filename)
        with open(file_path, "r") as f:
            return json.load(f)

    def mock_request(
        self,
        path: str,
        text: str = None,
        json_: dict = None,
        data_file: str = None,
        **kwargs,
    ):
        if text is not None:
            kwargs["text"] = text
        elif json_ is not None:
            kwargs["json"] = json_
        else:
            kwargs["json"] = self._load_data_file(data_file)
        self.requests.get(
            f"https://vulnerability.circl.lu/api/{path}",
            status_code=200,
            **kwargs,
        )

    def setUp(self):
        super().setUp()
        self.mock_http_session()

    def tearDown(self):
        self.cache.flushdb()
        return super().tearDown()

    def test_not_existing_vuln(self):
        self.mock_request("vulnerability/zeus", "null\n")

        self.input_message = INPUT
        self.run_bot()

        self.assertMessageEqual(0, INPUT)
        self.assertEqual(1, self.requests.call_count)

    def test_filter_classification(self):
        self.input_message = {**INPUT, "classification.type": "information-disclosure"}
        self.run_bot()

        self.assertMessageEqual(
            0, {**INPUT, "classification.type": "information-disclosure"}
        )
        self.assertEqual(0, self.requests.call_count)

    def test_no_epss(self):
        self.mock_request(
            "vulnerability/cve-2023-49606", data_file="CVE-2023-49606.json"
        )
        self.mock_request("epss/cve-2023-49606", json_={"data": []})

        self.input_message = {**INPUT, "classification.identifier": "CVE-2023-49606"}
        self.run_bot()

        expected = {
            **INPUT,
            "classification.identifier": "CVE-2023-49606",
            "event_description.url": "https://vulnerability.circl.lu/vuln/cve-2023-49606",
            "event_description.text": "A use-after-free vulnerability exists in the HTTP Connection Headers parsing in Tinyproxy 1.11.1 and Tinyproxy 1.10.0. A specially crafted HTTP header can trigger reuse of previously freed memory, which leads to memory corruption and could lead to remote code execution. An attacker needs to make an unauthenticated HTTP request to trigger this vulnerability.",  # noqa: E501
            "extra.cvss3_1": 9.8,
        }
        self.assertEqual(2, self.requests.call_count)
        self.assertMessageEqual(0, expected)

    def test_cve_with_epss_and_cut_description(self):
        self.mock_request(
            "vulnerability/CVE-2024-23113", data_file="CVE-2024-23113.json"
        )
        self.mock_request("epss/CVE-2024-23113", data_file="CVE-2024-23113_epss.json")

        self.input_message = {**INPUT, "classification.identifier": "CVE-2024-23113"}
        self.run_bot(parameters={"description_length": 100})

        expected = {
            **INPUT,
            "classification.identifier": "CVE-2024-23113",
            "event_description.url": "https://vulnerability.circl.lu/vuln/cve-2024-23113",
            "event_description.text": "A use of externally-controlled format string in Fortinet FortiOS versions 7.4.0 through 7.4.2, 7.2.0...",  # noqa: E501
            "extra.cvss3_1": 9.8,
            "extra.epss": "0.416590000",
        }
        self.assertEqual(2, self.requests.call_count)
        self.assertMessageEqual(0, expected)

    def test_cache_is_used(self):
        self.mock_request(
            "vulnerability/CVE-2024-23113", data_file="CVE-2024-23113.json"
        )
        self.mock_request("epss/CVE-2024-23113", data_file="CVE-2024-23113_epss.json")

        self.input_message = [
            {**INPUT, "classification.identifier": "CVE-2024-23113"},
            {**INPUT, "classification.identifier": "CVE-2024-23113"},
        ]
        self.run_bot(parameters={"description_length": 100}, iterations=2)

        expected = {
            **INPUT,
            "classification.identifier": "CVE-2024-23113",
            "event_description.url": "https://vulnerability.circl.lu/vuln/cve-2024-23113",
            "event_description.text": "A use of externally-controlled format string in Fortinet FortiOS versions 7.4.0 through 7.4.2, 7.2.0...",  # noqa: E501
            "extra.cvss3_1": 9.8,
            "extra.epss": "0.416590000",
        }
        self.assertEqual(2, self.requests.call_count)
        self.assertMessageEqual(0, expected)
        self.assertMessageEqual(1, expected)

    def test_multiple_cvss_and_description_with_new_lines(self):
        self.mock_request("vulnerability/cve-2025-2742", data_file="CVE-2025-2742.json")
        self.mock_request("epss/cve-2025-2742", json_={"data": []})

        self.input_message = {**INPUT, "classification.identifier": "CVE-2025-2742"}
        self.run_bot(parameters={"description_length": 100})

        expected = {
            **INPUT,
            "classification.identifier": "CVE-2025-2742",
            "event_description.url": "https://vulnerability.circl.lu/vuln/cve-2025-2742",
            "event_description.text": "A vulnerability classified     as critical was found in zhijiantianya ruoyi-vue-pro 2.4.1. This vuln...",  # noqa: E501
            "extra.cvss3_0": 5.4,
            "extra.cvss3_1": 5.4,
            "extra.cvss4_0": 5.3,
        }
        self.assertEqual(2, self.requests.call_count)
        self.assertMessageEqual(0, expected)

    def test_ghsa(self):
        self.mock_request(
            "vulnerability/ghsa-r6xg-mjqp-qqg4", data_file="GHSA-r6xg-mjqp-qqg4.json"
        )
        self.mock_request("epss/ghsa-r6xg-mjqp-qqg4", json_={"data": []})

        self.input_message = {
            **INPUT,
            "classification.identifier": "GHSA-r6xg-mjqp-qqg4",
        }
        self.run_bot(parameters={"description_length": 100})

        expected = {
            **INPUT,
            "classification.identifier": "GHSA-r6xg-mjqp-qqg4",
            "event_description.url": "https://vulnerability.circl.lu/vuln/ghsa-r6xg-mjqp-qqg4",
            "event_description.text": "A vulnerability classified as critical was found in zhijiantianya ruoyi-vue-pro 2.4.1. This vulnerab...",  # noqa: E501
        }
        self.assertEqual(2, self.requests.call_count)
        self.assertMessageEqual(0, expected)



"""Client to access the BlackKite library

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import logging
import math
from enum import Enum
from typing import Iterator, Sequence

import requests

from intelmq_extensions.lib.api_helpers import (
    DEFAULT_REFRESH_WINDOW,
    OAuthAccessMixin,
    RateLimiter,
)

from ....lib.blackkite import Category

default_logger = logging.getLogger(__name__)


class Status(str, Enum):
    ACTIVE = "Active"
    FALSE_POSITIVE = "FalsePositive"
    SUPPRESSED = "Suppressed"
    ACKNOWLEDGED = "Acknowledged"
    DELETED = "Deleted"


class Severity(str, Enum):
    INFO = "Info"
    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"
    CRITICAL = "Critical"


class Output(str, Enum):
    INFO = "Info"
    PASSED = "Passed"
    WARNING = "Warning"
    FAILED = "Failed"


_DEFAULT_STATUSES = [Status.ACTIVE]
_DEFAULT_SEVERITY = [Severity.CRITICAL]

CATEGORIES_WITH_OUTPUT = [
    Category.DNSHealth,
    Category.ApplicationSecurity,
    Category.EmailSecurity,
    Category.NetworkSecurity,
    Category.DDoSResiliency,
    Category.SSLTLSstrength,
    Category.InformationDisclosure,
]


class BlackKiteClient(OAuthAccessMixin):
    def __init__(
        self,
        url: str,
        client_id: str,
        client_secret: str,
        refresh_before: str = DEFAULT_REFRESH_WINDOW,
        session: requests.Session = None,
        logger: logging.Logger = default_logger,
        limit_requests: int = 60,
        limit_period: int = 60,
        page_size: int = 100,
    ) -> None:
        self.url = url
        self.logger = logger
        self._session = session
        self._page_size = page_size

        self.limiter = RateLimiter(limit_requests, limit_period)
        self.init_oauth(
            oauth_url=f"{url}/oauth/token",
            oauth_clientid=client_id,
            oauth_clientsecret=client_secret,
            session=session,
            logger=logger,
            refresh_before=refresh_before,
            limiter=self.limiter,
        )

    def get(self, path: str, params: dict = None, raw: bool = False):
        with self.limiter.call():
            response = self._session.get(
                f"{self.url}/{path}",
                params=params,
                headers={"Authorization": f"Bearer {self.access_token}"},
            )
        if not response.ok:
            self.logger.error(
                "Request %s failed with error %s, message: %s.",
                path,
                response.status_code,
                response.text,
            )
            raise RuntimeError(f"Request to {path} failed with {response.status_code}")
        return response if raw else response.json()

    def get_paginated(self, path: str, params: dict = None) -> Iterator[dict]:
        last = False
        page = 1
        params = params or {}
        while not last:
            params.update({"page_number": page, "page_size": self._page_size})
            response = self.get(path, params, raw=True)

            total_items = int(response.headers.get("X-Total-Items", "0"))
            last = page >= math.ceil(total_items / self._page_size)

            for element in response.json():
                yield element
            page += 1

    def list_findings(
        self,
        path: str,
        company_id: int,
        severities: Sequence[Severity] = None,
        statuses: Sequence[Status] = None,
        outputs: Sequence[Output] = None,
    ) -> Iterator[dict]:
        severities = severities or _DEFAULT_SEVERITY
        statuses = statuses or _DEFAULT_STATUSES

        params = {"status": ",".join(statuses), "severity": ",".join(severities)}
        if outputs:
            params["output"] = ",".join(outputs)

        return self.get_paginated(f"companies/{company_id}/findings/{path}", params)

    def status(self) -> dict:
        return self.get("status")

    def companies(self) -> Iterator[dict]:
        return self.get_paginated("companies")

    def get_findings_from_category(
        self,
        category: Category,
        company_id: int,
        severities: Sequence[Severity] = None,
        statuses: Sequence[Status] = None,
        outputs: Sequence[Output] = None,
    ):
        if category not in CATEGORIES_WITH_OUTPUT:
            outputs = None
        return self.list_findings(
            category.name.lower(), company_id, severities, statuses, outputs
        )

    def acknowledge_finding(self, company_id: int, finding_id: int):
        with self.limiter.call():
            result = self._session.patch(
                f"{self.url}/companies/{company_id}/findings/{finding_id}",
                headers={"Authorization": f"Bearer {self.access_token}"},
                json={"Status": Status.ACKNOWLEDGED.value},
            )
            self.logger.debug("ACK finding %s, result: %s.", finding_id, result.text)



"""Testing events splitter

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import unittest

from intelmq_extensions.bots.experts.event_splitter.expert import EventSplitterExpertBot

from ....base import BotTestCase

INPUT = {
    "__type": "Event",
    "classification.identifier": "zeus",
    "classification.type": "infected-system",
    "notify": False,
    "source.asn": 1,
    "source.ip": "192.0.2.1",
    "feed.name": "Example Feed",
}


class TestEventSplitterExpertBot(BotTestCase, unittest.TestCase):
    @classmethod
    def set_bot(cls):
        cls.bot_reference = EventSplitterExpertBot
        cls.sysconfig = {
            "look_in": "extra.tag",
            "copy_to": ["classification.identifier", "extra.vulnerabilities"],
            "regex": r"(cve-\d\d\d\d-[\d]+)",
        }

    def test_no_tags_sent_original_message(self):
        self.input_message = INPUT

        self.run_bot()

        self.assertOutputQueueLen(1)
        self.assertMessageEqual(0, INPUT)

    def test_no_matched_tags_sent_original_message(self):
        message = {**INPUT, "extra.tag": "some,different,tags"}
        self.input_message = message

        self.run_bot()

        self.assertOutputQueueLen(1)
        self.assertMessageEqual(0, message)

    def test_only_one_matched_apply_copy_to(self):
        message = {**INPUT, "extra.tag": "cve-2023-00000"}
        self.input_message = message

        self.run_bot()

        self.assertOutputQueueLen(1)
        self.assertMessageEqual(
            0,
            {
                **message,
                "classification.identifier": "cve-2023-00000",
                "extra.vulnerabilities": "cve-2023-00000",
            },
        )

    def test_multiple_matched_tags(self):
        message = {
            **INPUT,
            "extra.tag": "cve-2023-00000,other,cve-2023-00001,some,tag,cve-2023-00002",
        }
        self.input_message = message

        self.run_bot()

        self.assertOutputQueueLen(3)
        self.assertMessageEqual(
            0,
            {
                **message,
                "classification.identifier": "cve-2023-00000",
                "extra.vulnerabilities": "cve-2023-00000",
            },
        )
        self.assertMessageEqual(
            1,
            {
                **message,
                "classification.identifier": "cve-2023-00001",
                "extra.vulnerabilities": "cve-2023-00001",
            },
        )

        self.assertMessageEqual(
            2,
            {
                **message,
                "classification.identifier": "cve-2023-00002",
                "extra.vulnerabilities": "cve-2023-00002",
            },
        )



"""Tests for API helpers

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import time
from datetime import datetime, timedelta
from unittest import TestCase

import requests
from dateutil.tz import UTC

from intelmq_extensions.lib.api_helpers import OAuthAccessMixin, RateLimiter

from .base import OAuthAccess_APIMockMixIn


class OAuthAccessMixinTestCase(OAuthAccess_APIMockMixIn, TestCase):
    def setUp(self) -> None:
        super().setUp()
        self.oauth_config = {
            "oauth_clientid": "client1",
            "oauth_clientsecret": "secret1",
            "oauth_url": "https://oauthip.example.at/v1",
            "oauth_scope": "my_scope",
            "oauth_grant_type": "my_type",
        }

        self.session = requests.Session()
        self.setup_oauth_mock(
            **self.oauth_config,
            session=self.session,
        )
        self.mixin = OAuthAccessMixin()
        self.mixin.init_oauth(session=self.session, **self.oauth_config)

    def test_get_access_token(self):
        self.assertIn(".xxx", self.mixin.access_token)
        self.assertEqual(1, self.auth_requests.call_count)

    def test_refresh_access_token(self):
        # the client has a valid token
        valid_token = self.gen_fake_token(datetime.now(tz=UTC) + timedelta(minutes=30))
        self.mixin._access_token = valid_token

        # current token is used without calling the IdP
        self.assertEqual(valid_token, self.mixin.access_token)
        self.assertEqual(0, self.auth_requests.call_count)

        # but when token is about to be expired, the new one is requested
        self.mixin._access_token = self.gen_fake_token(
            datetime.utcnow() + timedelta(minutes=5)
        )
        self.assertEqual(self.mocked_access_token, self.mixin.access_token)
        self.assertEqual(1, self.auth_requests.call_count)


class RateLimiterTestCase(TestCase):
    def setUp(self) -> None:
        self.limiter = RateLimiter(2, 1)

    def test_call_in_limit(self):
        counter = 0
        start = time.monotonic_ns()

        with self.limiter.call():
            counter += 1
        with self.limiter.call():
            counter += 1

        stop = time.monotonic_ns()

        self.assertEqual(2, counter)
        self.assertLess(stop - start, 10**9)

    def test_call_exceed_limit(self):
        counter = 0

        start = time.monotonic_ns()
        with self.limiter.call():
            counter += 1
        with self.limiter.call():
            counter += 1
        middle = time.monotonic_ns()
        with self.limiter.call():
            counter += 1
        stop = time.monotonic_ns()

        self.assertEqual(3, counter)
        self.assertGreater(stop - start, 10**9)
        # Ensure there was a waiting time between second and 3rd call
        self.assertGreater(stop - middle, middle - start)

    def test_call_no_limit(self):
        counter = 0
        limiter = RateLimiter(2, period_seconds=0)

        start = time.monotonic_ns()
        with limiter.call():
            counter += 1
        with limiter.call():
            counter += 1
        with limiter.call():
            counter += 1
        stop = time.monotonic_ns()

        self.assertEqual(3, counter)
        self.assertLess(stop - start, 10**9)

    def test_call_nested(self):
        """Ensure the limiter is safe to eventually nest"""
        counter = 0
        start = time.monotonic_ns()

        with self.limiter.call():
            counter += 1
            with self.limiter.call():
                counter += 1
                with self.limiter.call():
                    counter += 1

        stop = time.monotonic_ns()

        self.assertEqual(3, counter)
        self.assertGreater(stop - start, 10**9)



"""Client to access the DISP API

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import logging
from datetime import datetime
from urllib.parse import quote, urlencode

import requests

from intelmq_extensions.lib.api_helpers import DEFAULT_REFRESH_WINDOW, OAuthAccessMixin

_SCOPE = "https://gateway.disp.deloitte.com/.default"
_GRANT_TYPE = "client_credentials"


default_logger = logging.getLogger(__name__)


class DISPClient(OAuthAccessMixin):
    def __init__(
        self,
        api_url: str,
        auth_token: str,
        oauth_clientid: str,
        oauth_clientsecret: str,
        oauth_url: str,
        session: requests.Session,
        refresh_before: int = DEFAULT_REFRESH_WINDOW,
        logger: logging.Logger = default_logger,
    ) -> None:
        self.api_url = api_url
        self.auth_token = auth_token
        self._access_token = None
        self._session = session
        self._page_size = 10
        self.logger = logger

        self.init_oauth(
            oauth_clientid=oauth_clientid,
            oauth_clientsecret=oauth_clientsecret,
            oauth_url=oauth_url,
            oauth_scope=_SCOPE,
            oauth_grant_type=_GRANT_TYPE,
            logger=self.logger,
            refresh_before=refresh_before,
            session=self._session,
        )

    def _auth(self):
        return {
            "Authorization": f"Bearer {self.auth_token}",
            "OAuth": self.access_token,
        }

    def get(self, path: str, params: dict = None):
        response = self._session.get(
            f"{self.api_url}/{path}",
            params=params,
            headers=self._auth(),
        )
        if not response.ok:
            self.logger.error(
                "Request %s failed with error %s, message: %s",
                path,
                response.status_code,
                response.text,
            )
            raise RuntimeError(f"Request to {path} failed with {response.status_code}")
        return response.json()

    def post(self, path: str, params: dict = None) -> requests.Response:
        response = self._session.post(
            f"{self.api_url}/{path}", params=params, headers=self._auth()
        )
        if not response.ok:
            self.logger.error(
                "Request %s failed with error %s, message: %s",
                path,
                response.status_code,
                response.text,
            )
            raise RuntimeError(f"Request to {path} failed with {response.status_code}")
        return response

    def get_paginated(self, path: str, params: dict = None):
        last = False
        page = 0
        params = params or {}
        while not last:
            # TODO: Use 'nextLink'
            params.update({"page": page, "size": self._page_size})
            response = self.get(path, params)
            last = response.get("last", True)
            for element in response.get("content", []):
                yield element
            page += 1

    def incidents(
        self, after: datetime = None, only_unread: bool = False, query: str = None
    ):
        if not query:
            conditions = []
            if after:
                long_timestamp = int(after.timestamp() * 1000)
                conditions.append(f"validationDate > {long_timestamp}")
            if only_unread:
                conditions.append("UNREAD")
            query = " AND ".join(conditions)
        # DISP rejects default encoding with + as space
        query = urlencode({"query": query}, quote_via=quote)

        return self.get_paginated(f"incident/?{query}")

    def download_evidence_json(self, incident_id: str, file_id: str):
        return self.get(f"incident/{incident_id}/file/{file_id}")

    def mark_incident_read(self, incident_id: str):
        self.post("incident/read", params={"id": incident_id, "read": True})



"""Base for testing APIs with OAuth token

SPDX-FileCopyrightText: 2023 CERT.at GmbH <https://cert.at/>
SPDX-License-Identifier: AGPL-3.0-or-later
"""

import json
from base64 import urlsafe_b64encode
from datetime import datetime, timedelta
from urllib.parse import parse_qs

import requests
from requests_mock import MockerCore


def _get_form_matcher(expected: dict):
    def _matcher(request: requests.Request):
        sent_data = parse_qs(request.body)
        for key, value in sent_data.items():
            if len(value) == 1:
                sent_data[key] = value[0]
        return expected == sent_data

    return _matcher


class OAuthAccess_APIMockMixIn:
    def setup_oauth_mock(
        self,
        oauth_clientid: str = "client1",
        oauth_clientsecret: str = "secret1",
        oauth_url: str = "https://oauthip.example.at/v1",
        oauth_scope: str = "",
        oauth_grant_type: str = "client_credentials",
        session: requests.Session = None,
    ):
        self._oauth_clientid = oauth_clientid
        self._oauth_clientsecret = oauth_clientsecret
        self._oauth_url = oauth_url
        self._oauth_scope = oauth_scope
        self._oauth_grant_type = oauth_grant_type
        self.__session = session
        self._mock_access_token()

    def _mock_access_token(self):
        self.auth_requests = MockerCore(session=self.__session, real_http=True)
        self.auth_requests.start()
        self.addCleanup(self.auth_requests.stop)

        expected = {
            "scope": self._oauth_scope,
            "client_id": self._oauth_clientid,
            "client_secret": self._oauth_clientsecret,
            "grant_type": self._oauth_grant_type,
        }
        expected = dict(filter(lambda kv: kv[1], expected.items()))
        self.mocked_access_token = self.gen_fake_token()

        def _gen_response(*_, **__):
            return {
                "token_type": "Bearer",
                "expires_in": 3599,
                "ext_expires_in": 3599,
                "access_token": self.mocked_access_token,
            }

        # TODO: matcher

        self.auth_requests.post(
            url=self._oauth_url,
            json=_gen_response,
            additional_matcher=_get_form_matcher(expected),
        )

    @staticmethod
    def gen_fake_token(expiry: datetime = None):
        expiry = expiry or (datetime.utcnow() + timedelta(hours=1))
        data = urlsafe_b64encode(
            json.dumps({"exp": int(expiry.timestamp())}).encode()
        ).decode()
        return f"xxxxx.{data}.xxx"



import json
from typing import Optional

from intelmq.lib.bot import ExpertBot
from intelmq.lib.mixins import CacheMixin
from intelmq.lib.utils import create_request_session

CACHE_FORMAT = "vuln:{identifier}"
CACHE_NOT_FOUND = "VULN_NOT_FOUND"


class VulnerabilityLookupExpertBot(ExpertBot, CacheMixin):
    url = "https://vulnerability.circl.lu"
    vulnerability_field = "classification.identifier"
    description_length = 500
    overwrite = False

    filter_classification_type = ["vulnerable-system"]

    redis_cache_ttl = 86400  # 1 day

    def init(self):
        self.set_request_parameters()
        self.http_session = create_request_session(self)

    def _get_vulnerability_data(self, vuln_id: Optional[str]) -> Optional[dict]:
        vuln_id = (vuln_id or "").strip().lower()
        if not vuln_id:
            return None

        cache_key = CACHE_FORMAT.format(identifier=vuln_id)
        cached_data = self.cache_get(cache_key)
        if cached_data:
            if cached_data == CACHE_NOT_FOUND:
                return None
            return json.loads(cached_data)

        response = self.http_session.get(f"{self.url}/api/vulnerability/{vuln_id}")
        if response.status_code != 200:
            response.raise_for_status()

        vuln_raw_data = response.json()
        if response.status_code == 404 or not vuln_raw_data:
            self.cache_set(cache_key, CACHE_NOT_FOUND)
            return None

        epss = None
        response = self.http_session.get(f"{self.url}/api/epss/{vuln_id}")
        if response.status_code == 200:
            epss_data = response.json().get("data", [])
            if epss_data:
                epss = epss_data[0].get("epss", None)
        else:
            self.logger.info(
                "Cannot get EPSS score, status code: %d", response.status_code
            )

        vuln_data = {"url": f"{self.url}/vuln/{vuln_id}"}
        if epss:
            vuln_data["epss"] = epss

        description = None
        cvss3_1 = None
        cvss3_0 = None
        cvss4_0 = None

        # Every source queried by Vulnerability Lookup provides different data format

        # CVE records
        if vuln_raw_data.get("dataType", "") == "CVE_RECORD":
            cna = vuln_raw_data.get("containers", {}).get("cna")
            if cna:
                for item in cna.get("descriptions", []):
                    if item.get("lang", "") in ["en"]:
                        description = item.get("value", "")
                        break

                for item in cna.get("metrics", []):
                    if "cvssV3_1" in item:
                        cvss3_1 = item["cvssV3_1"].get("baseScore")
                    if "cvssV3_0" in item:
                        cvss3_0 = item["cvssV3_0"].get("baseScore")
                    if "cvssV4_0" in item:
                        cvss4_0 = item["cvssV4_0"].get("baseScore")

        # GitHub Security Advisories
        if vuln_id.startswith("ghsa-"):
            description = vuln_raw_data.get("details")
            # TODO: Calculate CVSS

        if description:
            # Some CSV readers do not understand multi-line texts
            description = description.replace("\n", " ")
            vuln_data["description"] = description[: self.description_length]
            if len(description) > self.description_length:
                vuln_data["description"] += "..."

        if cvss3_0:
            vuln_data["cvss3_0"] = cvss3_0

        if cvss3_1:
            vuln_data["cvss3_1"] = cvss3_1

        if cvss4_0:
            vuln_data["cvss4_0"] = cvss4_0

        self.cache_set(cache_key, json.dumps(vuln_data))
        return vuln_data

    def process(self):
        event = self.receive_message()

        if (
            not self.filter_classification_type
            or event.get("classification.type") in self.filter_classification_type
        ):
            vuln_id = event.get(self.vulnerability_field)
            vuln_data = self._get_vulnerability_data(vuln_id) or {}

            if description := vuln_data.get("description"):
                event.add(
                    "event_description.text", description, overwrite=self.overwrite
                )

            if url := vuln_data.get("url"):
                event.add("event_description.url", url, overwrite=self.overwrite)

            for score_type in ["cvss3_0", "cvss3_1", "cvss4_0", "epss"]:
                if score := vuln_data.get(score_type):
                    event.add(f"extra.{score_type}", score, overwrite=self.overwrite)

        self.send_message(event)
        self.acknowledge_message()


BOT = VulnerabilityLookupExpertBot
